"""Execute a *nix command  and capture the output."""

import os
import re

__version__ = "$Revision: 1.5 $"
_comment = re.compile("\\s*#");


def getall(s):
	"""Read a list of lines from a process, after
	dropping junk like comments (beginning with whitespace then '#')
	or blank lines.
	Raises an exception if if the process returns
	a line beginning with 'ERR:'
	The argument s is a string containing the command and its 
	arguments.  S is normally passed to a shell."""

	pipe = os.popen(s, 'r')

	if pipe is None:
		raise RuntimeError, 'Cannot spawn pipe for {%s}'%s

	x = []
	while 1:
		line = pipe.readline();
		if line == '' :
			break
		if line.startswith('#'):
			continue
		cs = _comment.search(line)
		if cs is not None :	# Strip off the comment.
			line = cs.string[:cs.start()]
		line = line.rstrip()
		if line == '':
			continue
		if line.startswith('ERR:'):
			raise RuntimeError, '%s from {%s}' % (line, s)
		x.append(line.strip())
	
	sts = pipe.close()
	if sts is None: sts = 0
	if sts != 0 :
		raise RuntimeError, \
			'spawned command fails with %d from {%s}' % (sts, s)
	return x


def get(s):
	"""Read a single line from a process, after
	dropping junk like comments (#) or blank lines.
	Returns a trouble indication if the process returns
	'ERR:' """
	x = getall(s)
	if len(x) < 1:
		raise RuntimeError, 'no output fron {%s}' % s
	return x[0]



"""Find a set of points so that the distances between them match
a desired set of distances."""

import opt
import Numeric
import math
import RandomArray


EPS = 1e-3
G = 1

def _fcn(p, args, diffparam, diffctr, processor):
	assert p.shape[0] == G + args['np']*args['ndim']
	alpha = p[0]
	np = args['np']
	r = Numeric.zeros((np, np), Numeric.Float)
	c = Numeric.reshape(p[G:], (np, args['ndim']))
	dij = args['dij']
	dba = math.pow(args['dbar'], 1.0-alpha)
	print "_fcn, alpha=", alpha, "diffparam=", diffparam, "dba=", dba

	# Which residuals do I have to recalculate, if I'm changing parameter diffparam? :
	if diffparam is None or diffparam<G:
		rng = range(np)
	else:
		# rng = [ (diffparam-G)%np ] 	# --or--
		rng = [ (diffparam-G)/args['ndim'] ]
		r[:,:] = diffctr[:,:]
		print "# changing p", diffparam-G, "to", p[diffparam]

	# print "r.shape=", r.shape, "c.shape=", c.shape, "rng=", rng
	for i in range(np):
		for j in rng:
			r[i, j] = dba*(math.pow(_dist(c[i], c[j]), alpha)) - dij[i,j]
	if len(rng) < np:
		for i in rng:
			for j in range(np):
				r[i, j] = dba*(math.pow(_dist(c[i], c[j]), alpha)) - dij[i,j]

	ctrzero = Numeric.sum(c, axis=0)/(args['dbar']*math.sqrt(np))
	assert c.shape == (args['ndim'],)
	# Rotations, too! For first point, all but one coordinate is zero.  For second, all but two...
	rots = []
	for i in range(1, min(np, args['ndim'])):
		for j in range(args['ndim'] - i):
			rots.append(c[i-1, j])

	return (r, ctrzero)



def find_rep(dij, ndim):
	np = dij.shape[0]
	assert dij.shape == (np, np)

	dbar = math.sqrt(Numeric.sum(dij**2)/(np*np))
	print "# dbar=", dbar, "np=", np, "ndim=", ndim

	T = dbar*dbar/30

	nprm = np*ndim + G
	p = Numeric.zeros((nprm,), Numeric.Float)
	p[0] = 1.0
	p[G:] = RandomArray.random(np*ndim)

	o = opt.opt(p, T/20, _fcn, args=locals().copy())
	o.set_nthreads(1)
	o.working_precision = Numeric.Float32
	o.verbose = 1
	o.run(Ta=T)
	return (o.p.p[0], Numeric.reshape(o.p.p[G:], (np, ndim)))


def _dist(a, b):
	delta = a-b
	return Numeric.sum(delta**2)


def distances(xi):
	nx = xi.shape[0]
	nd = xi.shape[1]
	print "# nx=", nx, "nd=", nd
	assert xi.shape == (nx, nd)
	dij = Numeric.zeros((nx, nx), Numeric.Float)
	for i in range(nx):
		for j in range(nx):
			dij[i,j] = _dist(xi[i], xi[j])
	return dij


def test():
	y = Numeric.array([[-1,0], [1,0], [0,1], [0,-1]], Numeric.Float)
	print "y=", y
	dist = distances(y)
	print "dist=", dist
	alpha, x = find_rep(dist, 2)
	print "alpha=", alpha
	print "x=", x

if __name__ == '__main__':
	import gpkimgclass
	import sys
	import die
	arglist = sys.argv[1:]
	transpose = 0
	nd = 2
	while len(arglist)>0 and arglist[0].startswith('-'):
		arg = arglist.pop(0)
		if arg == '-d':
			nd = int(arglist.pop(0))
		elif arg == '-t':
			transpose = 1
		elif arg == '--':
			break
		else:
			die.die("Unrecognized argument: %s" % arg)
	print "# nd=", nd
	print "# transpose=", transpose
	f = arglist[0]
	data = gpkimgclass.read(f)
	if transpose:
		data.d = Numeric.transpose(data.d)
	dist = distances(data.d)
	alpha, x = find_rep(dist, nd)
	print "# alpha=", alpha
	assert x.shape[1] == 2
	for i in range(x.shape[0]):
		for j in range(nd):
			sys.stdout.write(" %g" % x[i,j])
		sys.stdout.write("\n")
		sys.stdout.flush()




"""When used as a script, this reads label files produced by wavesurfer
and prints the result.

Also contains functions that can be used for reading and writing the
label format preferred by Wavesurfer.
"""
import os
import re
import sys
from gmisclib.xwaves_errs import *

_avpat = re.compile('#\s*([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)\s*$')

_COMTAG = '_COMMENT'

def read(filename, loose=0):
	"""Read in label (transcription) files produced by wavesurfer.

	Note that leading or trailing spaces in the label are removed.

	@param filename: name of label file
	@type filename: str
	@param loose: how many minor deviations from the ideal format are allowed
	@type loose: int
	@return: (header, data).
		Data = [(starttime, endtime, label), ...].
	"""
	HUGE = 1e30
	hdr = {}
	if filename == '-':
		fd = sys.stdin
	else:
		try:
			fd = open(filename, "r")
		except IOError, x:
			raise NoSuchFileError, x
	n = 0
	t0last = -HUGE
	t1last = -HUGE
	d = []
	comments = []
	while True:
		n += 1
		l = fd.readline()
		if not l:
			break
		l = l.lstrip().rstrip('\r\n')
		if not l:
			if loose > 0:
				loose -= 1
			else:
				raise BadFileFormatError, 'blank lines prohibited: %s:%d' % (filename, n)
		if l == '#\n':
			continue
		if l.startswith('#'):
			m = _avpat.match(l)
			if m:
				hdr[m.group(1)] = m.group(2).strip()
			else:
				comments.append( l[1:].strip() )
			continue
		a = l.split(None, 2)
		if len(a) == 2 and float(a[0]) and float(a[1]):
			if l[-1] in (' ', '\t'):
				a.append( '' )
			elif loose > 0:
				a.append( '' )
				loose -= 1
			else:
				raise BadFileFormatError, 'label must not be empty: %s:%d' % (filename, n)
		if len(a) == 3:
			try:
				t0 = float(a[0])
				t1 = float(a[1])
			except ValueError:
				raise BadFileFormatError, 'Need <float> <float> before <label> %s:%d' % (filename, n)

			if t0>=t0last and t1>=t1last and t1>=t0:
				d.append( (t0, t1, a[2]) )
				t0last = t0
				t1last = t1
			else:
				raise DataOutOfOrderError, '%s:%d' % (filename, n)

	fd.flush()
	os.fsync(fd.fileno())	# Commit to disk.
	fd = None	# This will close most files, except for sys.stdin
			# which has probably some other reference keeping
			# it open.

	hdr[_COMTAG] = '\n'.join(comments)
	hdr['_NAME'] = filename
	hdr['_FILETYPE'] = 'wavesurfer'
	hdr['NAXIS'] = 2
	hdr['NAXIS2'] = len(d)
	hdr['NAXIS1'] = 3
	hdr['TTYPE1'] = 'start time'
	hdr['TUNIT1'] = 's'
	hdr['TTYPE2'] = 'end time'
	hdr['TUNIT2'] = 's'
	hdr['TTYPE3'] = 'label'

	return (hdr, d)


from xwaves_lab import end_marks, start_stop


def write(fd, header, data):
	"""Write label information to a file.
	Note: Expects data in [(t0, t1, label), ...] form.
	@param fd: where to write
	@param header: header information.
	@type header: dict
	@type fd: file or file-like object
	@param data: a listing of the segments to write to the file.
	@type data: [(segment_start_time, segment_end_time, segment_label), ...]
	"""
	HUGE = 1e30
	d = list(data)
	d.sort()

	fd.write('#wavesurfer_label_gpk\n')
	if _COMTAG in header:
		for x in header[_COMTAG].split('\n'):
			fd.write('# %s\n' % x)
	alist = header.items()
	alist.sort()
	for (a, v) in alist:
		if a != _COMTAG:
			fd.write('# %s = %s\n' % (a, v))
	fd.flush()
	t0last = -HUGE
	t1last = -HUGE
	for (i, (t0, t1, mark)) in enumerate(data):
		if t1 < t0 or t0<t0last or t1<t1last:
			raise DataOutOfOrderError, 'data[%d]: (%s, %s, %s)' % (i, t0, t1, mark)
		fd.write('%.5f %.5f %s\n' % (t0, t1, str(mark).strip()))
	fd.flush()


if __name__ == '__main__':
	hdr, data = read(sys.argv[1])
	for (t0, t1, lbl) in data:
		print t0, t1, lbl




import string
import Num

inttype = type(1)


class sbd:
	def __init__(self, sz, bw, _data = None):
		"""Constructs a symmetric band-diagonal matrix of sz*sz, with
		a nonzero bandwidth of bw.
		bw==0 corresponds to a diagonal matrix."""
		self.n = sz
		if bw > sz:
			raise ValueError, 'Bandwidth too large for size'
		self.kd = bw
		self.ldab = self.kd+1
		if _data is None:
			self.d = Num.zeros((self.n, self.ldab), Num.Float)
		else:
			self.d = _data	# Not a public interface!
		self.shape = (sz, sz)


	def __copy__(self):
		"""Copy the data, not just the data description."""
		return sbd(self.n, self.kd, Num.array(self.d, copy=True))

	def __deepcopy(self, m):
		return self.__copy__()


	def __idx2(self, key):
		"""Converts from (x,y) external representation of the
		matrix storage to the index where it is actually
		stored internally."""
		if len(key) != 2 or type(key[0])!=inttype or type(key[1])!=inttype:
			raise TypeError, 'Need two integer indices'
		diagdist = abs(key[1] - key[0])
		minidx = min(key[1], key[0])
		if minidx < 0:
			raise IndexError, 'indices must be positive'
		if diagdist > self.kd:
			raise IndexError, 'out of band: %d/%d/(%d,%d)'%(diagdist, self.kd, self.d.shape[0], self.d.shape[1])
		return (minidx, diagdist)


	def __getitem__(self, key):
		"""Pulls an element of the array.  Key is a 2-tuple that specifies the
		'virtual' position in the array (i.e., as if the array were a full
		square matrix, rather than a band diagonal symmetric matrix."""
		try:
			k0, k1 = self.__idx2(key)
		except IndexError:
			return 0.0
		return self.d[k0, k1]

	def __setitem__(self, key, value):
		k0, k1 = self.__idx2(key)
		self.d[k0, k1] = value

	def increment(self, key, delta):
		"""Increment a single value
		(and its symmetric partner, if off the diagonal)."""
		k0, k1 = self.__idx2(key)
		self.d[k0,k1] = delta + self.d[k0, k1]

	def bd_increment(self, key, delta):
		"""Block diagonal increment."""
		assert len(delta.shape) == 2
		assert delta.shape[0]==delta.shape[1]
		n = delta.shape[0]
		for i in range(n):
			e = n - i
			s = key + i
			# print "i=", i, "n=", n, "s=", s, "e=", e
			# print "sd.shape=", self.d[s,:e].shape
			# print "d.shape=", delta[i,i:].shape
			wrk = self.d[s,:e]
			Num.add(wrk, delta[i,i:], wrk)

	def __str__(self):
		o = []
		o.append( "<sbdarray\n")
		for i in range(self.n):
			o.append('[')
			for j in range(self.n):
				o.append( repr(self.__getitem__((i, j))) )
			o.append(']\n')
		o.append('>')
		return string.join(o, ' ');

	def __repr__(self):
		return self.__str__()

	def __pow__(self, other):
		return sbd(self.n, self.kd, (self.d)**other)

	def __getslice__(self, i, j):
		"""This is copy semantics, not shared reference.
		"""
		if i<0 or j<0:
			raise IndexError, 'indices must be positive'
		high = min(j, self.n)
		low = min(i, j)
		if high == low:
			return None
		rv = Num.zeros((high-low, self.n), Num.Float)
		for r in range(low, high):
			e = min(r+self.kd+1, self.n)
			rv[r-low,r:e] = self.d[r, 0:e-r]
		for r in range(max(0, low-self.kd), high):
			# Clunky, but correct:
			e = min(r+self.kd+1, self.n)
			ee = min(e, high)-low
			q = max(low-r, 0)
			rv[q+r-low:ee,r] = self.d[r, q:ee-(r-low)]
		return rv


def symmetrize(matrix, direction):
	return matrix


class NoSolutionError(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)


def solve(a, b0):
	""" solves a*x = b, where a is class sbd:
	symmetric, positive definite, and band-diagonal.
	Note that this destroys the contents of a."""
	b = Num.asarray(b0, Num.Float)
	if len(b.shape) == 1:
		# b = Num.array((b,), Num.Float)
		b = Num.reshape(b, (1, b.shape[0]))
	# print "b.shape=", b.shape
	assert a.n == b.shape[1]
	import lapack_dpb
	# print "type(a.d)==", type(a.d)
	# print "a.d=", a.d
	# print "ravel(a.d)=", Num.ravel(a.d)
	# print "a.ldab=", a.ldab
	result = lapack_dpb.dpbsv('L', a.n, a.kd, b.shape[0], a.d,
					a.ldab, b, max(1, a.n), 0)
	# print "result=", result
	if result['info'] != 0:
		raise NoSolutionError, 'Linear system has no solution. Lapack_dpb.dpbsv info code=%d' % result['info']
	# print "b=", b
	return b
 

def multiply(a, x):
	"""Calculates a*x, where a is class sbd:
	symmetric, positive definite, and band-diagonal."""
	xx = Num.asarray(x, Num.Float)
	assert len(xx.shape) == 1
	assert a.n == xx.shape[0]
	import dblas
	y = Num.zeros(xx.shape, Num.Float)
	# print "type(a.d)==", type(a.d)
	# print "a.d=", a.d
	# print "ravel(a.d)=", Num.ravel(a.d)
	# print "a.ldab=", a.ldab
	result = dblas.dsbmv('L', a.n, a.kd,
				Num.ones((1,), Num.Float), a.d,
				a.ldab, xx, 1,
				Num.zeros((1,), Num.Float),
				y, 1)
	assert result == 0
	# print "result=", result
	# print "b=", b
	return y


def test1():
	x = sbd(100, 10)
	x[0,0] = 1
	assert x[0,0] == 1
	x[99,99] = 2
	assert x[99,99] == 2
	x[10,0] = 10
	assert x[10,0] == 10
	assert x[0,10] == 10
	try:
		x[12,1] = 2
	except IndexError:
		pass
	else:
		raise RuntimeError, "whoops"


def err(a, b):
	return Num.sum((a-b)**2)

def test2():
	x = sbd(6,2)
	b = Num.zeros((6,), Num.Float) + 10
	for i in range(6):
		x[i,i] = 2
	x[2,1] = 0.5
	# print "x=", x
	y = solve(x, b)
	y_t = Num.array(([5, 4, 4, 5, 5, 5],))
	# print "y=", y, "y_t=", y_t
	# print "x=", x
	assert err(y_t, y) < 1e-9


def testm():
	a = sbd(6, 2)
	a[4,3] = 1
	a[4,4] = 2
	a[5,3] = 3
	a[5,5] = -1
	x = Num.array((1, 0, 0, 0, 0, 0), Num.Float)
	print multiply(a, x)
	x = Num.array((0, 0, 0, 0, 1, 0), Num.Float)
	print multiply(a, x)
	x = Num.array((0, 0, 0, 0, 0, 1), Num.Float)
	print multiply(a, x)


def testa():
	x = sbd(6, 2)
	x[4,3] = 1
	x[4,4] = 2
	x[5,3] = 3
	x[5,5] = -1

	y = x[:]
	# print "y=", y
	assert y[4,3]==1
	assert y[4,4]==2
	assert y[5,3]==3
	assert y[4,5]==0
	assert y[3,4]==1
	assert y[3,3]==0
	assert y[4,5]==0
	assert y[5,5] == -1
	y = x[5:6]
	# print "y=", y
	assert y[0,5] == -1
	assert y[0,3] == 3
	assert y[0,4] == 0

def testbdi():
	x = sbd(6,2)
	y = Num.array([[1, 2], [2, 3]])
	x.bd_increment(1, y)
	assert x[1,1]==1
	assert x[2,2]==3
	assert x[2,1]==2
	assert x[1,2]==2
	assert x[0,0]==0
	assert x[3,3]==0
	assert x[3,2]==0
	z = Num.array([[-1, 0, 1], [0, 0, 0], [0, 0, 0]])
	x.bd_increment(0, z)
	assert x[0,0]==-1
	assert x[0, 2]==1
	assert x[1,1]==1
	assert x[1, 2]==2

if __name__ == '__main__':
	test1()
	test2()
	testa()
	testbdi()
	testm()



# Copyright (c) 1999-2000 Gary Strangman; All Rights Reserved.
#
# This software is distributable under the terms of the GNU
# General Public License (GPL) v2, the text of which can be found at
# http://www.gnu.org/copyleft/gpl.html. Installing, importing or otherwise
# using this module constitutes acceptance of the terms of this License.
#
# Disclaimer
# 
# This software is provided "as-is".  There are no expressed or implied
# warranties of any kind, including, but not limited to, the warranties
# of merchantability and fittness for a given application.  In no event
# shall Gary Strangman be liable for any direct, indirect, incidental,
# special, exemplary or consequential damages (including, but not limited
# to, loss of use, data or profits, or business interruption) however
# caused and on any theory of liability, whether in contract, strict
# liability or tort (including negligence or otherwise) arising in any way
# out of the use of this software, even if advised of the possibility of
# such damage.
#
# Comments and/or additions are welcome (send e-mail to:
# strang@nmr.mgh.harvard.edu).
# 
import math
import Numeric as N

def fprob (dfnum, dfden, F):
    """
Returns the (1-tailed) significance level (p-value) of an F
statistic given the degrees of freedom for the numerator (dfR-dfF) and
the degrees of freedom for the denominator (dfF).

Usage:   lfprob(dfnum, dfden, F)   where usually dfnum=dfbn, dfden=dfwn
"""
    p = betai(0.5*dfden, 0.5*dfnum, dfden/float(dfden+dfnum*F))
    return p


def betai(a,b,x):
    """Returns the incomplete beta function::

	I-sub-x(a,b) = 1/B(a,b)*(Integral(0,x) of t^(a-1)(1-t)^(b-1) dt)

	where a,b>0 and B(a,b) = G(a)*G(b)/(G(a+b)) where G(a) is the gamma
	function of a.  The continued fraction formulation is implemented here,
	using the betacf function.  (Adapted from: Numerical Recipies in C.)

	Usage:   lbetai(a,b,x)
"""
    if (x<0.0 or x>1.0):
        raise ValueError, 'Bad x in lbetai'
    if (x==0.0 or x==1.0):
        bt = 0.0
    else:
        bt = math.exp(gammln(a+b)-gammln(a)-gammln(b)+a*math.log(x)+b*
                      math.log(1.0-x))
    if (x<(a+1.0)/(a+b+2.0)):
        return bt*betacf(a,b,x)/float(a)
    else:
        return 1.0-bt*betacf(b,a,1.0-x)/float(b)




def betacf(a,b,x):
    """
This function evaluates the continued fraction form of the incomplete
Beta function, betai.  (Adapted from: Numerical Recipies in C.)

Usage:   lbetacf(a,b,x)
"""
    ITMAX = 200
    EPS = 3.0e-7

    bm = az = am = 1.0
    qab = a+b
    qap = a+1.0
    qam = a-1.0
    bz = 1.0-qab*x/qap
    for i in range(ITMAX+1):
        em = float(i+1)
        tem = em + em
        d = em*(b-em)*x/((qam+tem)*(a+tem))
        ap = az + d*am
        bp = bz+d*bm
        d = -(a+em)*(qab+em)*x/((qap+tem)*(a+tem))
        app = ap+d*az
        bpp = bp+d*bz
        aold = az
        am = ap/bpp
        bm = bp/bpp
        az = app/bpp
        bz = 1.0
        if (abs(az-aold)<(EPS*abs(az))):
            return az
    print 'a or b too big, or ITMAX too small in Betacf.'


def gammln(xx):
    """
Returns the gamma function of xx.
    Gamma(z) = Integral(0,infinity) of t^(z-1)exp(-t) dt.
(Adapted from: Numerical Recipies in C.)

Usage:   lgammln(xx)
"""

    coeff = [76.18009173, -86.50532033, 24.01409822, -1.231739516,
             0.120858003e-2, -0.536382e-5]
    x = xx - 1.0
    tmp = x + 5.5
    tmp = tmp - (x+0.5)*math.log(tmp)
    ser = 1.0
    for j in range(len(coeff)):
        x = x + 1
        ser = ser + coeff[j]/x
    return -tmp + math.log(2.50662827465*ser)


def agammln(xx):
    """
Returns the gamma function of xx.
    Gamma(z) = Integral(0,infinity) of t^(z-1)exp(-t) dt.
Adapted from: Numerical Recipies in C.  Can handle multiple dims ... but
probably doesn't normally have to.

Usage:   agammln(xx)
"""
    coeff = [76.18009173, -86.50532033, 24.01409822, -1.231739516,
             0.120858003e-2, -0.536382e-5]
    x = xx - 1.0
    tmp = x + 5.5
    tmp = tmp - (x+0.5)*N.log(tmp)
    ser = 1.0
    for j in range(len(coeff)):
        x = x + 1
        ser = ser + coeff[j]/x
    return -tmp + N.log(2.50662827465*ser)




#!/usr/bin/env python

import Num
import math


class boxc(object):
	def __init__(self, xc, yc, xw, yw):
		self._xc = xc
		self._yc = yc
		self._xw = xw
		self._yw = yw
	
	def xmin(self):
		return self._xc - self._xw/2.0

	def xmax(self):
		return self._xc + self._xw/2.0

	def ymin(self):
		return self._yc - self._yw/2.0

	def ymax(self):
		return self._yc + self._yw/2.0
	
	def w(self):
		return self._xw
	
	def h(self):
		return self._yw

	def yc(self):
		return self._yc

	def xc(self):
		return self._xc

	def shift(self, x, y):
		return boxc(self._xc+x, self._yc+y, self._xw, self._yw)


class text_template(object):
	def __init__(self, text, H, W):
		"""(0,0) is the lower left corner of the block of text."""
		self.lines = []
		self.txt = []
		la = text.split('\n')
		n = len(la)
		for (i, l) in enumerate(la):
			self.addline(0.0, H*(n-1-i), W*len(l), H, l)
	
	def addline(self, x0, y0, xlength, yht, txtline):
		"""Specify LLC."""
		self.lines.append(boxc(x0+xlength/2.0, y0+yht/2.0,
					xlength, yht)
					)
		self.txt.append(txtline)

	def nlines(self):
		return len(self.lines)


def Gauss(y, ybar, yvar):
	arg = (y-ybar)**2/(2*yvar)
	return Num.exp(-Num.minimum(arg, 100.0))/math.sqrt(2*math.pi*yvar)

def gauss(y, ybar, yvar):
	arg = (y-ybar)**2/(2*yvar)
	return math.exp(-arg)/math.sqrt(2*math.pi*yvar)

def q_gauss(y, ybar, yvar):
	return gauss(y, ybar, yvar) - 0.3*gauss(y, ybar, 40*yvar)
	


class viewport(boxc):
	def __init__(self, fracCharHt, fracCharW, xmin, xmax, ymin, ymax):
		self.H = fracCharHt
		self.W = fracCharW
		self.datasets = []
		self.avoidlist = []
		boxc.__init__(self, xmin, ymin, xmax-xmin, ymax-ymin)
		self.YN = 100
		self.DF = 10.0
	

	def dataset(self, x, y):
		self.datasets.append( (x, y) )


	def _fom_y(self, ycand, x0, relbox):
		rlist = []
		xtra_var = self.h()**2 * ((1.0/self.YN)**2 + self.H**2)
		xbdr = self.h() * self.H
		for (x, y) in self.datasets :
			ovlap = Num.greater(x, x0+relbox.xmin()-xbdr) * Num.less(x, x0+relbox.xmax()+xbdr)
			avoid = Num.compress(ovlap, y)
			an = avoid.shape[0]
			if an > 1:
				asum = Num.sum(avoid, axis=0)
				assq = Num.sum((avoid-asum/an)**2, axis=0)
				assq += an * xtra_var
				# Avoid the data in the relevant x-range:
				rlist.append( (asum/an, assq/an, -self.DF) )
				# But be nearby:
				rlist.append( (asum/an, 100.0*assq/an, 0.1*self.DF) )
	
			rn = y.shape[0]
			if rn > 1:
				rsum = Num.sum(y, axis=0)
				rssq = Num.sum((y-rsum/rn)**2, axis=0)
				rssq += rn * xtra_var
				# Also be near the overall dataset
				rlist.append( (rsum/rn, 2*rssq/rn, 0.1*self.DF) )

		for a in self.avoidlist:
			veff = relbox.w()**2 + a.w()**2
			wt = q_gauss(x0+relbox.xc(), a.xc(), veff)
			rlist.append( (a.yc(), a.h()**2, -wt) )

		fom = Num.zeros(ycand.shape, Num.Float)
		yco = ycand + relbox.ymin()
		for (rbar, rvar, wt) in rlist:
			fom += wt*Gauss(yco, rbar, rvar+relbox.h()**2)
		return fom
	
	def find_y(self, x0, ttm):
		"""Figures out a good place to put a label.
		The label is at a specified tc, but can be at any vertical
		position.   This finds a good vertical position.
		Returns the bottom of the block of text.
		"""
		delta = self.h()/self.YN
		y_candidates = (Num.arrayrange(self.YN)-(0.5*self.YN))*delta + self.yc()
		fom = Num.zeros(y_candidates.shape, Num.Float)
		for lbox in ttm.lines:
			tmp = self._fom_y(y_candidates, x0, lbox)
			Num.add(fom, tmp, fom)
		return y_candidates[Num.argmax(fom)]


	def textplace(self, x0, text):
		tt = text_template(text, self.H*self.h(), self.W*self.w())
		yc = self.find_y( x0, tt )
		for (l, bx) in zip(tt.txt, tt.lines):
			bxs = bx.shift(x0, yc)
			self.avoidlist.append(bxs)
			yield (bxs.xmin(), bxs.ymin(), l)
	

	def draw_hor_line(self, xc, yc, w):
		EPS = 0.005
		self.avoidlist.append(boxc(xc, yc, w, EPS*self.h()))




"""Subroutines for q_classifier.py that implement a multi-mu model.
That is, one where all the classes share a common covariance matrix.
"""

import math
import multivariance as M
import q_classifier as QC
from gmisclib import dictops

HUGE = 1e30

def count_the_classes(data):
	"""How many examples of each class do we have?"""
	counts = {}
	for datum in data:
		dictops.add_doc(counts, datum.classid, 1)
	return counts



def build_classifiers(data, N, modelchoice=M.multi_mu):
	"""Builds a set of quadratic classifiers.
	Data is a list of datum_tr .
	It returns a list: [ (evaluation_result, classifier_description), ... ] ."""

	assert modelchoice==M.multi_mu or modelchoice==M.multi_mu_diag

	cprms = M.meanvar(data, N, modelchoice)
	assert isinstance(cprms[0], M.multi_mu_with_numbers)	\
		or isinstance(cprms[0], M.multi_mu_diag_with_numbers)

	classlist = cprms[0].desc.int_to_id.values()

	if modelchoice==M.multi_mu_diag:
		details = M.diag_quadratic(ndim=cprms[0].desc.ndim())
	elif modelchoice==M.multi_mu:
		details = M.quadratic(ndim=cprms[0].desc.ndim())
	else:
		RuntimeError, 'Cannot happen'

	classcounts = count_the_classes(data)
	# Now, we build a bunch of classifiers
	classifiers = []
	for tmp in cprms:
		qc = QC.classifier()
		for c in classlist:
			# Bias is the Bayesean prior: classes with more training examples
			# are more likely.
			bias = math.log(classcounts[c])
			if modelchoice==M.multi_mu_diag:
				q = M.diag_quadratic_with_numbers(tmp.mu[c], tmp.invsigma, \
										details, bias)
			elif modelchoice==M.multi_mu:
				q = M.quadratic_with_numbers(tmp.mu[c], tmp.invsigma,
										details, bias)
			else:
				RuntimeError, 'Cannot happen'
			qc.addQuad(c, q)
		classifiers.append(qc)
	return classifiers





class DontHashThis(Exception):
	"""If your trimmer funcion raises this exception, then the input
			to nice_hash.add() will be ignored."""

	def __init__(self):
		Exception.__init__(self)


class NotInHash(KeyError):
	def __init__(self, s):
		KeyError.__init__(self, s)




class simple(object):
	__doc__ = """This class generates a map from inputs to integers,
		where an equivalence class is defined by the trimmer function.
		It is just a stripped-down version of the L{hash} class
		defined above.   Faster and less memory consumption, if
		you don't need the rmap method.
		"""



	def __init__(self, trimmer=None):
		"""
		@param trimmer: Defines equivalence classes on list;
			it can be some sort of projection operation.
			The default (None) makes the identity class equal to the inputs.
			It can also specify which inputs to ignore by raising the
			L{DontHashThis} exception.
		"""
		self.trimmed_seen = {}
		self.n = 0
		if trimmer is None:
			def trimmer(x):
				return x
		self.trimmer = trimmer


	def add(self, x):
		"""Returns an integer which is shared among all x in the
		equivalence class, but different from all other x.
		If the trimmer function raises the DontHashThis exception,
		it will ignore the input and return None.
		@returns: int
		"""

		try:
			xt = self.trimmer(x)
		except DontHashThis:
			return None

		try:
			i = self.trimmed_seen[xt]
		except KeyError:
			i = self.n
			self.trimmed_seen[xt] = i
			self.n += 1
		return i


	def add_newonly(self, x):
		"""Similar to add, except it returns None if the equivalence
		class has already appeared.
		@returns: int or None
		"""

		try:
			xt = self.trimmer(x)
		except DontHashThis:
			return None

		if xt in self.trimmed_seen:
			return None

		i = self.n
		self.trimmed_seen[xt] = i
		self.n += 1
		return i
		

	def get_image(self, x):
		"""Returns an integer which is shared among all x in the
		equivalence class, but different from all other x.
		"""

		try:
			xt = self.trimmer(x)
		except DontHashThis:
			raise NotInHash('Trimmer raised DontHashThis')
		try:
			i = self.trimmed_seen[xt]
		except KeyError:
			raise NotInHash('Key: %s' % str(xt))
		return i


	def map(self):
		"""Return the map from classes (as returned by the trimmer function)
		to integers."""
		return self.trimmed_seen.copy()


	def classes(self):
		"""Return a list over all the classes that have been seen."""
		return list(self.trimmed_seen.keys())


	def classmap(self):
		"""Return the map from integers to classes.
		@return: [classname, classsname, ...]
		"""
		o = [None] * len(self.trimmed_seen)
		for (x, i) in self.trimmed_seen.iteritems():
			o[i] = x
		return  o


class hash(simple):
	def __init__(self, trimmer=None):
		"""This adds the L{rmap} method to the L{simple} class.
		"""
		simple.__init__(self, trimmer)
		self.seen = []


	def rmap(self):
		"""Returns the map from integers to inputs."""
		# o = [ [] for i in range(len(self.trimmed_seen)) ]
		o = [ [] for q in self.trimmed_seen.keys() ]
		for (x, cl) in self.seen:
			o[self.trimmed_seen[cl]].append(x)
		return o


	def test(self):
		map = self.map()
		rmap = self.rmap()
		for (intg, inpt) in rmap.iteritems():
			for s in inpt:
				assert map[s] == intg

	def add(self, x):
		"""Returns an integer which is shared among all x in the
		equivalence class, but different from all other x.
		If the trimmer function raises the DontHashThis exception,
		it will ignore the input and return None.
		Note that if you add an item twice, it will appear twice in rmap().
		@returns: int
		"""

		try:
			xt = self.trimmer(x)
		except DontHashThis:
			return None
		self.seen.append((x, xt))

		try:
			i = self.trimmed_seen[xt]
		except KeyError:
			i = self.n
			self.trimmed_seen[xt] = i
			self.n += 1
		return i


	def add_newonly(self, x):
		"""Similar to add, except it returns None if the equivalence
		class has already appeared.

		The list of classes seen (self.seen) then
		contains only the first item that was added in each class,
		so L{rmap}() will produce a dictionary containing single-item lists:
		C{{int1:[item1],int2:[item2],int3:[item3], ...}}.
		Calls to L{add}() and L{add_newonly}() can be intermixed safely,
		though the result of L{rmap}() will then not contain all added items.
		@returns: int or None
		"""

		try:
			xt = self.trimmer(x)
		except DontHashThis:
			return None

		if xt in self.trimmed_seen:
			return None

		self.seen.append((x, xt))
		i = self.n
		self.trimmed_seen[xt] = i
		self.n += 1
		return i



nice_hash = hash


def map(list, trimmer=lambda x:x):
	h = simple(trimmer)
	for x in list:
		h.add(x)
	return h.map()



if __name__ == '__main___':
	h = nice_hash(lambda x : x)
	assert h.add('foo') == 0
	assert h.add('bar') == 1
	assert h.add('gleep') == 2
	assert h.add('bar') == 1
	h.test()
	assert h.map()['bar'] == 1
	assert h.rmap()[0] == ['foo']
	assert h.classmap()[1] == 'bar'



from gmisclib import Num

"""Perceptual frequency scales."""
__version__ = "$Revision: 1.3 $"

if hasattr(Num, 'asinh'):
	asinh = Num.asinh
else:
	def asinh(x):
		tmp = Num.greater(x, 0)
		p = Num.log(x + Num.sqrt(x*x + 1))
		n = -Num.log(-x + Num.sqrt(x*x + 1))
		return tmp*p + (1-tmp)*n

def acosh(x):
	assert not Num.greater(x, 1)
	return Num.log(x - Num.sqrt(x*x-1))

def f_to_bark(f):
	"""frequency to critical band number.
	See Schroeder et al. (1979).
	"""
	return 7*asinh(Num.asarray(f)/650.0)

def bark_to_f(b):
	"""critical band number -> frequency
	M. R. Schroeder, B.S. Atal, J.L. Hall (1979), J.Acous.Soc.Am. 66(6) 1647-1652.
	Title: "Optimizing Digital Speech Coders by exploiting masking
	properties of the Human Ear."
	"""
	return 650*Num.sinh(Num.asarray(b)/7.0)


def cbw(b):
	"""Critical bandwidth (in Hz) at frequency b (barks)."""
	return (650.0/7.0)*Num.cosh(b/7.0)


if __name__ == "__main__":
	print f_to_bark(51)
	print f_to_bark(149)



"""This a helper module for multivariance.py"""

import Num
import multivariance_classes as M
import random
import g_implements

class datum_c:
	def __init__(self, vector, classid):
		self.classid = classid
		self.value = vector


def _multi_mu__init__(self, dataset=None, ndim=None, idmap=None, details=None):
		"""You either give it a complete dataset to look at,
		including class IDs, *or* the dimensionality of the data
		(ndim) and a map between classids and integers.
		This map can be obtained from nice_hash.
		"""
		import dictops

		if dataset is not None:
			assert details is None
			assert ndim is None and idmap is None
			import nice_hash
			assert (ndim is None) and (idmap is None)
			h = nice_hash.nice_hash(lambda x:x)
			for t in dataset:
				assert g_implements.impl(t, datum_c), \
					"Bad input type: %s" % g_implements.why(t, datum_c)
				h.add(t.classid)
			ndim = len(dataset[0].value)
			idmap = h.map()
		elif details is not None:
			assert ndim is None and idmap is None
			ndim = details.ndim()
			idmap = details.id_to_int
		assert (ndim is not None) and (idmap is not None)
		self.Nmu = len(idmap)
		self.id_to_int = idmap.copy()
		self.int_to_id = dictops.rev1to1(idmap)
		# print 'MM__init__id_to_int=', self.id_to_int
		# print 'MM__init__int_to_id=', self.int_to_id
		for i in range(len(self.int_to_id)):
			assert i in self.int_to_id, "Not a good mapping to indices."


class multi_mu(M.modeldesc):
	__doc__ = """This describes a quadratic model of a known size,
			with multiple means
			(one for each different class of data)."""

	def __init__(self, dataset=None, ndim=None, idmap=None, details=None):
		M.modeldesc.__init__(self, ndim)
		_multi_mu__init__(self, dataset, ndim, idmap, details)
	__init__.__doc__ = _multi_mu__init__.__doc__


	def modeldim(self):
		m = self.ndim()
		return self.Nmu*m + (m*(m+1))/2


	def unpack(self, prms):
		m = self.ndim()
		assert len(prms) == self.modeldim()
		mu = {}
		for i in range(self.Nmu):
			mu[self.int_to_id[i]] = prms[i*m:(i+1)*m]
		# print 'MMunpackMu=', mu
		invsigma = Num.zeros((m, m), Num.Float)
		j = self.Nmu*m
		for i in range(m):
			tmp = prms[j:j+(m-i)]
			invsigma[i,i:] = tmp
			invsigma[i:,i] = tmp
			j += m-i
		return self.new(mu, invsigma)


	def new(self, mu, invsigma, bias=0.0):
		"""Mu is a mapping of classids to vectors.  invsigma is a square matrix."""
		# print 'new(', mu, invsigma, ')'
		assert type(mu) == type({})
		# print 'NewMu=', mu
		return multi_mu_with_numbers(mu, invsigma, self, bias)


	def start(self, data):
		raise RuntimeError, 'Broken'
		import nice_hash
		h = nice_hash.nice_hash(lambda x: x.classid)
		for datum in data:
			assert g_implements.impl(datum, datum_c)
			h.add(datum)
		if len(data) > 1:
			ivar = M.diag_inv_variance([datum.value for datum in data])
		else:
			ivar = Num.identity(self.ndim())
		divar = Num.diagonal(ivar)
		rnd = {}
		var = {}
		for (k, v) in h.rmap().items():
			# print 'start: k=', k, 'v=', v, 'divar=', divar
			rnd[self.int_to_id[k]] = random.choice(v).value
			var[self.int_to_id[k]] = 1.0/divar
		# print 'ivarsize=', ivar.shape, Num.outerproduct(divar, divar).shape
		return (self.new(rnd, ivar),
			self.new(var, Num.outerproduct(divar,divar))
			)


class multi_mu_with_numbers(M.model_with_numbers):
	def __init__(self, mu, invsigma, details, bias=0.0, offset=None):
		"""self.mu, self.invsigma, and self._offset should be considered
		read-only for all users of this class."""
		assert isinstance(details, multi_mu)
		M.model_with_numbers.__init__(self, details, bias)
		self.mu = Num.array(mu, copy=True)
		# print 'MMmu=', mu
		# print 'invsigma.shape=', invsigma.shape
		self.invsigma = Num.array(invsigma)
		self._offset = offset

	def __str__(self):
		return '<multi_mu: mu=%s; invsigma=%s >' % (str(self.mu), str(self.invsigma))

	__repr__ = __str__

	addoff = M._q_addoff	# Will not be called if _offset is not None

	def pack(self):
		n = self.ndim()
		# print 'invsigma.shape=', self.invsigma.shape, 'n=', n
		assert self.invsigma.shape == (n,n)
		assert len(self.mu) == self.desc.Nmu
		tmp = []
		# print 'self.mu=', self.mu
		for i in range(self.desc.Nmu):
			tmp.append( self.mu[self.desc.int_to_id[i]] )
		for i in range(n):
			tmp.append(self.invsigma[i,i:])
		# print 'Pack tmp=', tmp
		return Num.concatenate(tmp)


	def logp(self, datum):
		delta = datum.value - self.mu[datum.classid]
		parab = Num.dot(delta, Num.matrixmultiply(self.invsigma,
							delta))
		if not parab >= 0.0:
			raise M.QuadraticNotNormalizable, "Not positive-definite"
		return -parab/2.0 +  self.offset() + self.bias



class multi_mu_diag(M.modeldesc):
	__doc__ = """This describes a quadratic model of a known size,
			with multiple means (one for each different class of data).
			The covariance matrix is shared and diagonal."""

	def __init__(self, dataset=None, ndim=None, idmap=None, details=None):
		M.modeldesc.__init__(self, ndim)
		_multi_mu__init__(self, dataset, ndim, idmap, details)
	__init__.__doc__ = _multi_mu__init__.__doc__


	def modeldim(self):
		m = self.ndim()
		return self.Nmu*m + m


	def unpack(self, prms):
		m = self.ndim()
		assert len(prms) == self.modeldim()
		mu = {}
		for i in range(self.Nmu):
			mu[self.int_to_id[i]] = prms[i*m:(i+1)*m]
		# print 'MMunpackMu=', mu
		j = self.Nmu*m
		invsigma = prms[j:]
		return self.new(mu, invsigma)


	def new(self, mu, invsigma, bias=0.0):
		"""Mu is a mapping of classids to vectors.  Invsigma is a vector."""
		assert type(mu) == type({})
		return multi_mu_diag_with_numbers(mu, invsigma, self, bias)


	def start(self, data):
		raise RuntimeError, 'Broken'
		import nice_hash
		h = nice_hash.nice_hash(lambda x: x.classid)
		for datum in data:
			assert g_implements.impl(datum, datum_c)
			h.add(datum)
		if len(data) > 1:
			divar = M.vec_inv_variance([datum.value for datum in data])
		else:
			divar = Num.ones((self.ndim(),), Num.Float)
		rnd = {}
		var = {}
		for (k, v) in h.rmap().items():
			# print 'start: k=', k, 'v=', v, 'divar=', divar
			rnd[self.int_to_id[k]] = random.choice(v).value
			var[self.int_to_id[k]] = 1.0/divar
		return (self.new(rnd, divar),
			self.new(var, divar*divar)
			)


class multi_mu_diag_with_numbers(M.model_with_numbers):
	def __init__(self, mu, invsigma, details, bias=0.0, offset=None):
		"""self.mu, self.invsigma, and self._offset should be considered
		read-only for all users of this class."""
		assert isinstance(details, multi_mu_diag)
		M.model_with_numbers.__init__(self, details, bias)
		self.mu = Num.array(mu)
		# print 'MMmu=', mu
		# print 'invsigma.shape=', invsigma.shape
		self.invsigma = Num.array(invsigma)
		self._offset = offset

	def __str__(self):
		return '<multi_mu_diag: mu=%s; invsigma=%s >' % (str(self.mu), str(self.invsigma))

	__repr__ = __str__

	addoff = M._d_addoff	# Will not be called if _offset is not None

	def pack(self):
		n = self.ndim()
		# print 'invsigma.shape=', self.invsigma.shape, 'n=', n
		assert self.invsigma.shape == (n,)
		assert len(self.mu) == self.desc.Nmu
		tmp = []
		# print 'self.mu=', self.mu
		for i in range(self.desc.Nmu):
			tmp.append( self.mu[self.desc.int_to_id[i]] )
		tmp.append( self.invsigma )
		# print 'Pack tmp=', tmp
		return Num.concatenate(tmp)


	def logp(self, datum):
		delta = datum.value - self.mu[datum.classid]
		parab = Num.sum(self.invsigma * delta**2)
		if not parab >= 0.0:
			raise M.QuadraticNotNormalizable, "Not positive-definite"
		return -parab/2.0 + self.offset() + self.bias







"""This is a helper module to make use of mcmc.py and mcmc_big.py.
It allows you to conveniently run a Monte-Carlo simulation of any
kind until it converges (L{stepper.run_to_bottom}) or until it
has explored a large chunk of parameter space (L{stepper.run_to_ergodic}).

It also helps you with logging the process.
"""

from __future__ import with_statement
import sys
import random
import thread as Thr
from gmisclib import Num
from gmisclib import g_implements
from gmisclib import mcmc

Debug = 0

class TooManyLoops(Exception):
	def __init__(self, *s):
		Exception.__init__(self, *s)


class warnevery(object):
	def __init__(self, interval):
		self.c = 0
		self.interval = interval
		self.lwc = 0
	
	def inc(self, incr=1):
		self.c += incr
		if self.c > self.lwc + self.interval:
			self.lwc = self.c
			return 1
		return 0

	def count(self):
		return self.c


class _counter(object):
	def __init__(self):
		self.lock = Thr.allocate_lock()
		self.count = 0

	def update(self, accepted):
		with self.lock:
			self.count += accepted

	def get(self):
		with self.lock:
			tmp = self.count
		return tmp



class _accum_erg(object):
	def __init__(self):
		self.lock = Thr.allocate_lock()
		self.erg = 0.0
		self.last_resetid = -1

	def update(self, x, nsteps):
		"""Increment an accumulator, but reset it to
		zero if the underlying stepper has called reset().
		The increment is by the ergodicity estimate, so
		C{self.erg} should approximate how many times the
		MCMC process had wandered across the probability
		distribution.
		"""
		with self.lock:
			tmp = x.reset_id()
			if tmp != self.last_resetid:
				self.last_resetid = tmp
				self.erg = 0.0
			else:
				self.erg += x.ergodic() * nsteps


	def get(self):
		with self.lock:
			tmp = self.erg
		return tmp


class logger_template(object):
	def add(self, stepperInstance, iter):
		pass

	def close(self):
		pass




class stepper(object):
	def __init__(self, x, maxloops=-1, logger=None):
		self.logger = None	# This sets things up for close()
		self.iter = 0
		self.maxloops = maxloops
		g_implements.check(x, mcmc.stepper)
		self.x = x
		if logger is None:
			self.logger = logger_template()
		else:
			g_implements.check(logger, logger_template)
			self.logger = logger


	def reset_loops(self, maxloops=-1):
		self.maxloops = maxloops


	def run_to_change(self, ncw=1, acceptable_step=None, update_T=False):
		"""Run the X{Markov Chain Monte-Carlo} until it finds
		an acceptable step.

		@param ncw: How many steps should be accepted before it stops?
		@type ncw: int
		@param acceptable_step: A function that decides what steps are acceptable and what are not.
			This is passed to the MCMC stepper, L{mcmc.BootStepper}.
		@type acceptable_step: function(float) -> bool, often L{mcmc.T_acceptor} or L{step_acceptor}
		@param update_T: Obsolete.   Must be False.
		@type update_T: boolean.
		@raise TooManyLoops: if it takes a long time before enough steps are accepted.
		"""
		assert not update_T, "Obsolete!"
		if acceptable_step is not None:
			old_acceptable = self.x.acceptable
			self.x.acceptable = acceptable_step
		else:
			old_acceptable = None
		n = 0
		self.communicate_hook("RTT:%d" % ncw)
		try:
			while n < ncw:
				n += self.x.step()
				self.logger.add(self.x, self.iter)
				if self.maxloops > 0:
					self.maxloops -= 1
				if self.maxloops == 0:
					raise TooManyLoops, "run_to_change"
				self.iter += 1
		finally:
			if old_acceptable is not None:
				self.x.acceptable_step = old_acceptable
			self.join('rtc end')
		return self.x.current()


	def communicate_hook(self, id):
		return


	def close(self):
		"""After calling close(), it is no longer legal
		to call run_to_change(), run_to_ergodic(), or
		run_to_bottom().    Logging is shut down, but the
		contents of the stepper are still available for
		inspection.
		"""
		if self.logger is not None:
			self.logger.close()
			self.logger = None


	# def __del__(self):
		# self.close()


	def _nc_get_hook(self, nc):
		return nc.get()


	def run_to_ergodic(self, ncw=1, T=1.0):
		"""Run the stepper until it has explored all of parameter space
		C{ncw} times (as best as we can estimate).   Note that this
		is a pretty careful routine.   If the stepper finds a better
		region of parameter space so that log(P) improves part way
		through the process, the routine will reset itself and begin
		again.

		NOTE: this sets C{T} and C{sortstrategy} for the L{stepper<mcmc.BootStepper>}.

		@param ncw: how many times (or what fraction) of an ergodic
			exploration to make.
		@type ncw: int.
		@param T: temperature at which to run the Monte-Carlo process.
			This is normally 1.0.   If it is C{None}, then the
			current temperature is not modified.
		@type T: C{float} or C{None}.
		@return: a L{position<mcmc.position_base>} at the end of the process.
		"""
		ISSMAX = 20
		self.join('rte before loop')
		if T is not None:
			acceptable_step = mcmc.T_acceptor(T)
		else:
			acceptable_step = None
		ss = self.x.set_sort_strategy(self.x.SSNEVER)
		nc = _accum_erg()
		try:
			iss = 1
			while True:
				ncg = self._nc_get_hook(nc)
				if ncg >= ncw:
					break
				elif ncg < 0.5*ncw and iss<ISSMAX:
					iss += 1
				# print 'ncg=', ncg, '/', ncw, 'iss=', iss
				self.run_to_change(ncw=iss, acceptable_step=acceptable_step)
				nc.update(self.x, iss)
		finally:
			self.x.set_sort_strategy(ss)	# Return to previous sortstrategy.
		# There will be trouble if you get an exception caused by maxloops.
		# There is yet no way to synchronize all MPI threads.
		self.join('rte after loop')
		return self.x.current()




	def _not_at_bottom(self, xchanged, nchg, es, dotchanged, ndot):
		return (Num.sometrue(Num.less(xchanged,nchg))
					or es<1.0 or dotchanged<ndot
					or self.x.acceptable.T()>1.5
				)


	def run_to_bottom(self, ns=3, acceptable_step=None, update_T=False):
		"""Run the X{Markov Chain Monte-Carlo} until it converges
		near a minimum.

		@param ns: This controls how carefully if checks that
			it has really found a minimum.   Large values
			will take longer but will assure more accurate
			convergence.
		@type ns: L{int}
		@param update_T: Permission to fiddle with the simulated
			annealing temperature.   If C{update_T} is False,
			C{self.x.T} is left with whatever value you
			originally gave it.   OBSOLETE.
			Use C{acceptable_step} instead.
		@type update_T: boolean.
		"""
		m = self.x.np
		if update_T and acceptable_step is None:	#Backwards compatibility
			acceptable_step = step_acceptor(n=m, T0=10.0, T=1.0, ooze=0.5)
		else:
			acceptable_step = self.x.acceptable
		nchg = int(round(ns * m**0.25))
		ndot = int(round((2*ns)**0.5))
		for i in range(2+m//2):
			self.run_to_change(5, acceptable_step=acceptable_step)	# get it warmed up.
		before = self.run_to_change(ns, acceptable_step=acceptable_step)
		mid = self.run_to_change(ns, acceptable_step=acceptable_step)
		es = 0.0
		lock = Thr.allocate_lock()
		xchanged = Num.zeros(m, Num.Int)
		dotchanged = 0
		last_resetid = self.x.reset_id()
		while self._not_at_bottom(xchanged, nchg, es, dotchanged, ndot):
			try:
				after = self.run_to_change(ns, acceptable_step=acceptable_step)
			except TooManyLoops, x:
				raise TooManyLoops, ('run_to_bottom: s' % x, self.iter, es, dotchanged, m,
							Num.sum(Num.less(xchanged, nchg)),
							self.x.current().prms()
							)
			# We keep track of direction reversals to see when
			# we reach the bottom.
			Num.add(xchanged,
				Num.less( (after.vec()-mid.vec())
						* (mid.vec()-before.vec()), 0.0),
				xchanged)

			dotchanged += Num.dot(after.vec()-mid.vec(),
						mid.vec()-before.vec()) < 0.0

			es += self.x.ergodic() * ns**0.5

			# print 'LA'
			lock.acquire()
			if self.x.reset_id() != last_resetid:
				# A new maximum: we are probably
				# not stable.  Likely, we are still in the
				# improvement phase.
				last_resetid = self.x.reset_id()
				lock.release()
				es = 0.0
				xchanged[:] = 0
				dotchanged = 0
				self.note('run_to_bottom: Reset: T=%g' % acceptable_step.T(), 1)
			else:
				lock.release()
				self.note('run_to_bottom: T=%g' % acceptable_step.T(), 3)

			before = mid
			mid = after
		self.join('rtb after loop')
		return self.iter


	def join(self, id):
		return

	def note(self, s, lvl):
		if Debug >= lvl:
			print s
			sys.stdout.flush()



class step_acceptor(object):
	def __init__(self, n, T0, T, ooze):
		ooze = float(ooze)
		assert 0.0 <= ooze < 1.0
		T = float(T)
		assert T > 0 and T0>0
		assert n > 0
		self._T = T
		self.ooze = ooze
		self.E = [random.expovariate(1.0)*T0 for i in range(n+1)]


	def T(self):
		sum = 0.0
		for e in self.E:
			sum += e
		return sum/len(self.E)


	def __call__(self, delta):
		# print 'delta=', delta, "E=", self.E
		n = len(self.E)
		i = random.randrange(n)
		ok = delta > -self.E[i]
		if not ok:
			j = random.randrange(n)
			if j != i:
				sum = self.E[i] + self.E[j]
				f = random.random()
				self.E[i] = f*sum
				self.E[j] = sum - self.E[i]
				# print 'Not OK E->', self.E
			return False
		self.E[i] = (delta + self.E[i])*self.ooze + self._T*(1.0-self.ooze)
		# print 'OK: updating E[%d] to %g' % (i, self.E[i])
		return True

	def __repr__(self):
		return '<step_acceptor(T=%g o=%g) %s>' % (self._T, self.ooze, str(self.E))



def is_root():
	"""This is a stub for compatibility with MPI code."""
	return True

def size():
	"""This is a stub for compatibility with MPI code."""
	return 1


def test():
	def test_logp(x, c):
		# print '#', x[0], x[1]
		return -(x[0]-x[1]**2)**2 + 0.001*x[1]**2
	# mcmc.Debug = 3
	# Debug = 3
	x = mcmc.bootstepper(test_logp, Num.array([0.0,2.0]),
				Num.array([[1.0,0],[0,2.0]]))
	thr = stepper(x)
	nsteps = thr.run_to_bottom(acceptable_step=step_acceptor(x.np, 5.0, 1.0, 0.5))
	print '#nsteps', nsteps
	assert nsteps < 300
	for i in range(2):
		print 'RTC'
		thr.run_to_change(2)
		print 'RTE'
		thr.run_to_ergodic(1.0)
		print 'DONE'
	thr.close()




if __name__ == '__main__':
	test()



#!/usr/bin/env python

import die
import fiatio
import g2_select
import dictops

DEBUG = 0

def merge(d1, d2, key1, key2, grab, unique=True):
	m1 = {}
	d = DEBUG
	for datum1 in d1:
		k = tuple( [datum1[t] for t in key1] )
		if d > 0:
			die.info('Sample datum key1: %s' % str(k) )
			d -= 1
		if unique and k in m1:
			raise ValueError, "Key=%s seen multiple times in d1" % str(k)
		dictops.add_dol(m1, k, datum1)
	m2 = {}
	d = DEBUG
	for datum2 in d2:
		k = tuple( [datum2[t] for t in key2] )
		if d > 0:
			die.info('Sample datum key2: %s' % str(k) )
			d -= 1
		if unique and k in m2:
			raise ValueError, "Key=%s seen multiple times in d2" % str(k)
		dictops.add_dol(m2, k, datum2)


	o = []
	for k in m1.keys():
		if k in m2:
			for a in m1[k]:
				for b in m2[k]:
					tmp = a.copy()
					for (g_from, g_to) in grab:
						tmp[g_to] = g2_select.evaluate(g_from, b)
					o.append( tmp )
	return o



if __name__ == '__main__':
	import sys
	arglist = sys.argv[1:]
	key = []
	unique = True
	grab = []
	while arglist and arglist[0].startswith('-'):
		arg = arglist.pop(0)
		if arg == '--':
			break
		elif arg == '-key':
			key1 = arglist.pop(0).split(',')
			key2 = key1
		elif arg == '-key1':
			key1 = arglist.pop(0).split(',')
		elif arg == '-key2':
			key2 = arglist.pop(0).split(',')
		elif arg == '-grab':
			gcode = arglist.pop(0)
			gout = arglist.pop(0)
			grab.append( (gcode, gout) )
		elif arg == '-D':
			DEBUG += 1
		elif arg == '-multi':
			unique = False
		else:
			die.die('Unknown flag: %s' % arg)
	h1, d1, c1 = fiatio.read(open(arglist.pop(0), 'r'))
	h2, d2, c2 = fiatio.read(open(arglist.pop(0), 'r'))
	assert len(arglist) == 0
	h2.update(h1)
	o = merge(d1, d2, key1, key2, grab, unique)
	fiatio.write(sys.stdout, o, c1 + c2, h2)



"""This module provides a way of safely specifing accent
positions in running text.

If you have a transcription "I did not eat the orange ball.",
you can attach a "+" symbol to the word "eat" like this:
"+eat".

You enter an accent specification, which consists of words with
a prefix (the prefix is anything that ends in a punctuation mark).
The program matches up the words in the accent specification
to the words in the transcription.
You can have many words in the accent specification, if necessary.
You can also disambiguate things by putting context words into
the accent spec (without a prefix).
All matching is done left-to-right.
"""

import re

__version__ = "$Revision: 1.4 $"

_goodprefix = re.compile("""^.*\W$""")
_goodsuffix = re.compile("""^\W.*$""")

class BadMatchError(RuntimeError):
	def __init__(self, x):
		RuntimeError.__init__(self, x)

def prefix(text_array, accent_spec, map_fcn=lambda x:x):
	"""This function takes an array of words and an accent spec.
	It matches the accent spec to the words,
	and outputs an array of tuples which tells you
	where the accents are, and what kind.
	The optional map_fcn can be used to map other kinds
	of objects into a array of strings.

	More specifically, an accent_spec is a whitespace-separated
	list of strings.   Each string is a word from the text_array,
	with an optional prefix.   The strings are matched in
	order to the words, and the output array is a list
	of (index_in_text_array, prefix_text) tuples.

	So, if you have text_array = ['my', 'cat', 'is', 'my', 'cat']
	and accent_spec="is +my", then align() will match
	"is" to text_array[2], and "+my" to text_array[3],
	and it will return [ (3, "+") ] .
	Note that "+is +my" does not imply that 'is' and 'my'
	are adjacent, and "+is +cat" simply returns [ (2, "+"), (4, "+") ].

	If the accent_spec were "+my", then it would match
	text_array[0], and return [ (0, "+") ].

	Prefixes can be multiple characters,
	but they cannot end in letters, digits, or underscore.
	"""


	asa = accent_spec.split()
	if len(asa) > len(text_array):
		raise BadMatchError, "Accent spec longer than text array: %d vs. %d" % (len(asa), len(text_array))
	as0 = asa.pop(0)
	out = []
	j = 0
	for t in map_fcn(text_array):
		# print "AS=", as0, "t=", t
		if as0 == t:	# A match for alignment purposes.
			# print "Align match"
			if len(asa) == 0:
				return out
			else:
				as0 = asa.pop(0)
		elif as0.endswith(t) and _goodprefix.match(as0[:-len(t)]):
				# A match with an accent.
			# print "Acc match:", as0[:-len(t)]
			out.append( (j, as0[:-len(t)]) )
			if len(asa) == 0:
				return out
			else:
				as0 = asa.pop(0)
		j += 1
	raise BadMatchError, "Accent_spec not consistent with text:  %d/%d." % (len(asa), len(text_array))




def suffix(text_array, accent_spec, map_fcn=lambda x:x):
	"""See prefix, but with the obvious changes."""

	asa = accent_spec.split()
	if len(asa) > len(text_array):
		raise BadMatchError, "Accent spec longer than text array: %d vs. %d" % (len(asa), len(text_array))
	as0 = asa.pop(0)
	out = []
	j = 0
	for t in map_fcn(text_array):
		# print "AS=", as0, "t=", t
		if as0 == t:	# A match for alignment purposes.
			# print "Align match"
			if len(asa) == 0:
				return out
			else:
				as0 = asa.pop(0)
		elif as0.startswith(t) and _goodsuffix.match(as0[len(t):]):
				# A match with an accent.
			# print "Acc match:", as0[:-len(t)]
			out.append( (j, as0[len(t):]) )
			if len(asa) == 0:
				return out
			else:
				as0 = asa.pop(0)
		j += 1
	raise BadMatchError, "Accent_spec not consistent with text:  %d/%d." % (len(asa), len(text_array))



def preshow(text_array, alignment, map_fcn=lambda x:x):
	"""Shows an alignment in a printable form.
	It puts the prefixes in the appropriate places.
	"""
	tm = [ map_fcn(t) for t in text_array ]
	for (idx, tag) in alignment:
		tm[idx] = tag + tm[idx]
	return tm



def sufshow(text_array, alignment, map_fcn=lambda x:x):
	"""Shows an alignment in a printable form.
	It puts the suffixes in the appropriate places.
	"""
	tm = [ map_fcn(t) for t in text_array ]
	for (idx, tag) in alignment:
		tm[idx] = tm[idx] + tag
	return tm




def test():
	ta = ['my', 'cat', 'is', 'my', 'cat']
	assert prefix(ta, "is +my") == [ (3, '+') ]
	assert prefix(ta, "+my") == [ (0, '+') ]
	assert suffix(ta, "my+") == [ (0, '+') ]
	assert preshow(['my', 'cat', 'Fred'], [(1, '!')]) == ['my', '!cat', 'Fred']
	assert sufshow(['my', 'cat'], [(1, '!')]) == ['my', 'cat!']


if __name__ == '__main__':
	test()
	import sys
	arglist = sys.argv[1:]
	pre = 1
	if len(arglist)>0 and arglist[0] == '-s':
		pre = 0
		arglist.pop(0)
	elif len(arglist)>0 and arglist[0] == '-p':
		pre = 1
		arglist.pop(0)
	textarray = arglist
	accspec = sys.stdin.readline().strip()
	if pre:
		alignment = prefix(textarray, accspec)
		print ' '.join(preshow(textarray, alignment))
	else:
		alignment = suffix(textarray, accspec)
		print ' '.join(sufshow(textarray, alignment))



#!/usr/bin/env python

"""When run as a script:
python ~/lib/wavio.py [-g gain] -wavin|-wavout infile outfile
Reads or writes .wav files from any format supported by
gpkimgclass.py .

As a library, allows reading of class gpk_img data from a .WAV
file, and writing class gpk_img to a .WAV file.
"""
import wave
import gpkimgclass
from gmisclib import Num

#: Cause L{write} to raise an error on overflow.
OV_ERROR = 0
#: Cause L{write} to truncate overflows.
OV_LIMIT = 1
#: Cause L{write} to silently ignore overflows.
OV_IGNORE = 2

_sizes = {
	2: (Num.int16, 16),
	4: (Num.int32, 32)
	}


def read(fn, tstart=None, t_end=None):
	"""Reads in a .WAV file and returns a L{gpkimgclass.gpk_img<gpkimgclass>} instance
	that contains the header and data information.   If tstart and/or
	t_end are specified, it seeks and only reads in the necessary data
	between tstart and t_end.
	@param fn: filename
	@type fn: str
	@param tstart: time at which to start reading audio data
	@param t_end: time at which to stop reading audio data
	@type tstart: float
	@type t_end: float
	@return: audio data
	@rtype: L{gpkimgclass} instance
	"""

	w = wave.open(fn, "r")
	assert w.getcomptype() == 'NONE', "Can't handle %s compression" % w.getcompname()
	nf = w.getnframes()
	nc = w.getnchannels()
	dt = 1.0/w.getframerate()
	numtype, bitpix = _sizes[w.getsampwidth()]
	if tstart is not None:
		istart = int(round(tstart/dt))
		w.setpos(istart)
	else:
		istart = 0
	if t_end is not None:
		nf = int(round(t_end/dt)) - istart
		if nf < 0:
			raise ValueError, "t_end=%.4f < tstart=%.4f" % (t_end, tstart)
	data = Num.fromstring(w.readframes(nf), numtype)
	w.close()
	data = Num.reshape(data, (nf, nc))
	hdr = {
		'NAXIS1': nc, 'NAXIS2': nf,
		'CDELT2': dt, 'CRPIX2': 1, 'CRVAL2': istart*dt,
		'BITPIX': bitpix
		}
	return gpkimgclass.gpk_img(hdr, data)


def _itemsize(z):
	"""This exists for Numeric/NumPy compatibility"""
	x = z.itemsize
	if isinstance(x, int):
		return x
	if callable(x):
		return x()
	raise RuntimeError, 'Cannot deal with %s' % str(type(x))

_typecodes = {
	_itemsize(Num.zeros((1,), Num.Int32))*8:
			(Num.Int32, 2.0**31-1.0, 1.0-2.0**31),
	_itemsize(Num.zeros((1,), Num.Int16))*8:
			(Num.Int16, 2.0**15-1.0, 1.0-2.0**15),
	_itemsize(Num.zeros((1,), Num.Int8))*8:
			(Num.Int8, 2.0**7-1.0, 1.0-2.0**7)
	}

def write(data, fname, scalefac=1, allow_overflow=OV_ERROR):
	"""@param data: is a class gpk_img object containing
		data to be written (note that the header information is
		ignored except for the sampling rate, bits per pixel,
		and number of channels.
	@param fname: is the name of a file to write it to (or a file object),
	@type fname: str or file,
	@param scalefac: is a factor to multiply the data
	@param allow_overflow: can be either
		- L{OV_ERROR} (default, means raise a ValueError exception
			if the data*scalefac overflows),
		- L{OV_LIMIT} (means limit the data*scalefac to prevent
			overflows -- this clips the audio), or
		- L{OV_IGNORE} (means let the overflows happen and don't worry.)
	"""
	if not (data.dt() > 0.0):
		raise ValueError, "Cannot set sampling rate: dt=%g\n" % data.dt()

	hdr = data.hdr

	if not hdr.has_key('BITPIX') or int(hdr['BITPIX'])==0:
		bitpix = 16
	else:
		bitpix = abs(int(hdr['BITPIX']))
	assert bitpix % 8 == 0
	if bitpix > 32:
		bitpix = 32

	tc, vmax, vmin = _typecodes[bitpix]
	dds = data.d * scalefac
	if allow_overflow == OV_ERROR:
		if not Num.alltrue(Num.greater_equal(Num.ravel(dds), vmin)):
			raise ValueError, "Scaled data overflows negative"
		if not Num.alltrue(Num.less_equal(Num.ravel(dds), vmax)):
			raise ValueError, "Scaled data overflows positive"
	elif allow_overflow == OV_LIMIT:
		dds = Num.clip(dds, vmin, vmax)
	else:
		assert allow_overflow == OV_IGNORE

	d = Num.around(dds).astype(tc).tostring()
	w = wave.open(fname, "w")
	w.setnchannels(data.d.shape[1])
	w.setnframes(data.d.shape[0])
	w.setsampwidth(bitpix/8)
	w.setframerate( int(round( 1.0/float(data.dt()) )) )
	w.writeframesraw(d)


if __name__ == '__main__':
	import sys
	arglist = sys.argv[1:]
	gain = None
	if len(arglist)==0:
		print __doc__
		sys.exit(1)
	if arglist[0] == '-g':
		arglist.pop(0)
		gain = float(arglist.pop(0))
	if arglist[0] == '-wavin':
		x = read(arglist[1])
		if gain is not None:
			Num.multiply(x.d, gain, x.d)
		x.write(arglist[2])
	elif arglist[0] == '-wavout':
		x = gpkimgclass.read(arglist[1])
		rxd = Num.ravel(x.d)
		mxv = max( rxd[Num.argmax(rxd)], -rxd[Num.argmin(rxd)] )
		print "# max=", mxv
		if gain is None:
			gain = 32000.0/mxv
		write(x, arglist[2], gain)
	else:
		print __doc__
		sys.exit(1)



import Num
import math

# import sys
# sys.path.append('/home/gpk/djt_code/dpss')
import djt_spectrum


def _p(v, width, ctr, w, K):
	s = int(round(ctr - width/2.0))
	if s < 0:
		return 0
	e = s + width
	if e >= len(v):
		return 0
	p, lmbda = djt_spectrum.low_pass_power(v[s:e], w, K)
	assert lmbda[0]>0.7 and lmbda[-1]>0.7
	# print "lambda=", lmbda
	return p



def pwr_guts(v, dt, binw, w, K):
	"""Local power in a signal.
	dt and binw are measured in terms of the sampling interval of v.
	w is the bandwidth of the filter.
	K is the order of the prolate speroidal fcn.
	"""
	assert binw >= 1
	v = Num.asarray(v)
	nb = int(round(len(v)/dt))
	o = Num.zeros((nb,), Num.Float)
	bw = int(round(binw))
	for i in range(nb):
		o[i] = _p(v, bw, dt*i, w, K)
	return o


def pwr(v, dt, bandwidth, K):
	w = bandwidth
	binw = 0.5*float(K)/float(w)
	return pwr_guts(v, dt, binw, w, K)

def expand(r, a):
	n = len(a)
	nn = int(round(r * n))
	o = Num.zeros((nn,), Num.Float)
	last = 0
	for i in range(n):
		end = int(round((i+1)*r))
		o[last : end] = a[i]
		last = end
	return o


def pwrs(dt, harmonics, f0, signal, sig_dt, bandwidth):
	"""This is a classical synchronous demodulator, with
	DPSS windowing to get maximally steep skirts and
	out-of-band rejection.

	dt=output power measurement interval.
	harmonics = [ which harmonics to measure (multiples of f0) ]
	f0 = frequency as a function of time (measured at intervals of dt)
	signal = input signal to be demodulated
	sig_dt = sampling interval of signal.
	"""

	K = 3
	assert len(f0.shape) == 1
	assert bandwidth > 0
	f = expand(dt/sig_dt, f0)
	phase = Num.add.accumulate((2*math.pi*sig_dt)*f)
	f = None	# Reclaim memory
	o = []
	signal = signal[:len(phase)]
	for h in harmonics:
		mod = Num.exp((1j * h) * phase) * signal
		o.append(pwr(mod, dt/sig_dt, bandwidth*sig_dt, K))
	return o


def one_ampl(harmonics, phase, signal, binw, nbin=None):
	"""Everything measured in samples.  Phase and signal have same sampling rate."""
	import sharp_energy
	if nbin is None:
		nbin = 4
	if harmonics == "?data length":
		return sharp_energy.one_median(binw, "?data length", None, nbin)
	assert phase.shape[0] == signal.shape[0]
	center = 0.5 * signal.shape[0]
	o = []
	for h in harmonics:
		mod = Num.exp((1j * h) * phase) * signal
		m = sharp_energy.one_median(binw, center, mod, nbin)
		o.append(m)
	return o


def stepped_ampl(dt, harmonics, f0, signal, sig_dt, subwindow=0.010, nsubw = 4):
	"""signal -> amplitudes in specified harmonics
	dt=time interval of f0 samples = time interval of ampl measurements.
	"""
	assert len(f0.shape) == 1
	import sharp_energy
	f = expand(dt/sig_dt, f0)
	phase = Num.add.accumulate((2*math.pi*sig_dt)*f)
	f = None # Reclaim memory.
	o = []
	signal = signal[:len(phase)]
	for h in harmonics:
		mod = Num.exp((1j * h) * phase) * signal
		m = sharp_energy.stepped_median(dt/sig_dt, subwindow/sig_dt,
						mod, nsubw)
		o.append(m)
	return o



def test0():
	for period in range(6,50):
		t = Num.arrayrange(1000)
		f = 1.0/period
		s = Num.cos(t*f*2*math.pi)
		x = stepped_ampl(100, [1, 2], Num.array([f]*5), s, 1, 100, 4)
		print period, abs(x[0][2]), x[0][2], abs(x[1][2])



def test1():
	o = []
	for period in range(2, 200):
		t = Num.arrayrange(1000)
		s = Num.cos(t*2*math.pi/period)
		o.append(pwr(s, 100.0, 0.01, 4)[3])
	for period in range(3, 200):
		print period, o[period-1]



if __name__ == '__main__':
	test0()
	# test1()



"""This module tells you if an object's signature matches a class.
It lets you know if one class can substitute for another,
even if they are unrelated in an OO hierarchy.

This allows you to begin to check that a foreign object will have the
necessary properties to be used in your code.   It is more of a
functional check than L{isimplements}().

For instance, if your code needs a write() method, you
can check for it like this::

class some_template(object):
	def write(self, somearg):
		pass
#
g_implements.check(x, some_template)
#... at this point, we can be assured that a
# call to x.write(3) is possible.
# (Note that we don't know what the call will *do*,
# merely that the proper method of x exists and that
# it can take an argument.


Semi-kluge:   if you have an instance, C{i} that needs to convince
C{g_implements} that it can actually behave as class C{C},
then set C{i.__g_implements__ = ['C']}.
Generally, this can be a list or set of class names.
This trick is useful for C-implementations of python classes
(where introspection cannot get all the necessary information) or
for fancy python classes that use C{__getattribute__} and similar.
"""

import inspect

_cache = {}
# g_implements keeps a cache of previous results for speed.  MAXCACHE
# sets the size of the number if types to remember.  Note that the classes
# are cached, not the instances themselves.   You can call L{why}(o, C)
# with a million different objects C{o}, and as long as all of them are
# in the same class, this will only occupy one cache slot.
# (And, the million queries will be answered from the cache as long as all
# of the C{o} are in the same class.)
MAXCACHE = 100

Optional = 'optional'
Varargs = 'varargs'
def Vartype(mem, com):
	return True
def Strict(mem, com):
	return False

Ignore = set( ['__class__', '__delattr__', '__module__', '__new__',
			'__doc__', '__dict__', '__getattribute__',
			'__hash__',
			'__repr__', '__str__',
			'__reduce__', '__reduce_ex__', '__setattr__',
			'__weakref__', '__slots__'
			]
		)


def why(instance, classobj):
	"""Explains why instance doesn't implement the classobj.
	@param instance: an instance of a class
	@param classobj: a class object
	@return: None (if it does) or a string explanation of why it doesn't
	@rtype None or str
	"""
	try:
		c = instance.__class__
	except AttributeError:
		return "Instance (arg 1) has no __class__ attribute."
	key = (c, classobj)
	try:
		return _cache[key]
	except KeyError:
		pass
	assert MAXCACHE >= 0
	while len(_cache) > MAXCACHE:
		_cache.popitem()

	try:
		# This is a semi-kluge to allow an instance to declare that it
		# can implement a particular class.
		if classobj.__name__ in instance.__g_implements__:
			_cache[key] = None
			return None
	except AttributeError:
		pass

	for mem in dir(classobj):
		if mem in Ignore:
			continue
		v = getattr(classobj, mem)
		gis = getattr(v, 'g_implements', Strict)
		if gis == Optional:
			continue
		if not hasattr(c, mem):
			_cache[key] = 'Missing member: %s; %s does not implement %s' % (mem, str(type(instance)), str(classobj))
			return _cache[key]
		com = getattr(c, mem)
		if callable(gis) and gis(mem, com):
			continue
		if inspect.ismethod(com):
			if inspect.ismethod(v):
				vargs, vv, vk, vdef = inspect.getargspec(v.im_func)
				margs, mv, mk, mdef = inspect.getargspec(com.im_func)
	
				if mdef is None:
					matchlen = len(margs)
				else:
					matchlen = len(margs)-len(mdef)
				if vargs[:matchlen] != margs[:matchlen]:
					if gis == Varargs:
						continue
					_cache[key] = "Method argument list does not match for %s: %s instead of %s; type=%s does not implement %s" \
								% (mem, margs[:matchlen], vargs[:matchlen],
									str(type(instance)), str(classobj))
					return _cache[key]
			elif inspect.ismethoddescriptor(v):
				continue
		elif type(com) != type(v):
			_cache[key] = 'Wrong type for %s: %s instead of %s; type=%s does not implement %s' \
					% (mem, type(com), type(v),
						str(type(instance)), str(classobj)
						)
			return _cache[key]
	_cache[key] = None
	return _cache[key]


def impl(instance, classobj):
	"""Tells you if an instance of an object implements a class.
	By implements, I mean that the instance supplies every member
	that the class supplies, and every member has the same type.
	The instance may have *more* members, of course.
	Functions require that the argument names must match, too
	at least as far as the required arguments in the classobj's function.

	The match may be made looser by adding a g_implements attribute
	to various class members.  Possibilities for the value
	are Optional, Strict, Vartype, Varargs, or you can give
	a two-argument function, and that function will be called
	to decide whether the match is acceptable or not.
	"""
	return why(instance, classobj) is None


def make_optional(x):
	"""This is a decorator for a function.
	make_optional implies make_varargs.
	"""
	x.g_implements = Optional
	return x

def make_varargs(x):
	"""This is a decorator for a function."""
	x.g_implements = Varargs
	return x

def make_vartype(x):
	"""This is a decorator for a function."""
	x.g_implements = Vartype
	return x

def make_strict(x):
	"""This is a decorator for a function."""
	x.g_implements = Strict
	return x


class GITypeError(TypeError):
	"""Exception raised by L{check} to indicate failure.
	"""
	def __init__(self, s):
		TypeError.__init__(self, s)


def check(instance, classobj):
	"""Require that C{instance} has the attributes defined by C{classobj}.
	This function either quietly succeeds or it raises a GITypeError exception.
	@type instance: any instance of a class (i.e. anything with a C{__class__} attribute).
	@type classobj: any class object (i.e. not an instance, typically).
	@param instance: the thing to check
	@param classobj: a class that provides a pattern of attributes.
	@raise: L{GITypeError}
	"""
	w = why(instance, classobj)
	if w is not None:
		raise GITypeError, w


class _tc1(object):
	x = 1
	y = ''
	def z(self, foo):
		return foo

class _tc2(object):
	def __init__(self):
		pass

class _tc1a(object):
	x = 0
	y = 'x'
	def z(self, foo):
		return foo+1

class _tc3(object):
	x = 0.0
	y = 'x'
	def z(self, foo):
		return foo+1

class _tc1b(object):
	x = 0
	y = 'x'
	def z(self, foo):
		return foo+1
	w = []
	def foo(self, a, b):
		return a+b


class _tc0a(object):
	x = 1

class _tc0b(object):
	y = ''
	def z(self, foo):
		return 0

class _tc1c(_tc0a, _tc0b):
	pass

class _tc2a(object):
	x = 1
	y = ''
	def z(self, bar):
		return bar

def test():
	assert not impl(_tc1(), _tc2)
	check(_tc1(), _tc1a)
	assert not impl(_tc1(), _tc3)
	assert not impl(_tc1(), _tc1b)
	check(_tc1b(), _tc1)
	check(_tc1c(), _tc1)
	check(_tc1(), _tc1c)
	assert not impl(_tc1(), _tc2a)

if __name__ == '__main__':
	test()



"""Fit a linear transform to a bunch of
input/output vectors."""


from gmisclib import Num
from gmisclib import gpk_lsq
from gmisclib import gpk_rlsq


def err_before_fit(data):
	"""How much variation did the data have before the fit?
	This is used to compare with the error after the fit, to
	allow a F-test or ANOVA.
	@return: Returns the per-coordinate sum-squared-error
		of the output coordinates. (float).
	@param data: [(input_coordinates, output_coordinates), ...] or
		[(input, output, weight), ...].
		A list of (in,out) tuples corresponding to the
		independent and dependent parameters of the linear transform.
		Both C{in} and C{out} are one-dimensional
		L{numpy vectors<numpy.ndarray>}.   They don't need to have
		the same length, though (obviously) all instances of "in"
		need to have the same length and all instances of "out"
		also need to match one another.   Weights are C{float}.
	@rtype data: [(L{numpy.ndarray}, L{numpy.ndarray}), ...]
	"""

	if len(data) < 2:
		return None

	m = None
	o = []
	for (ic, oc) in data:
		assert len(oc.shape) == 1, "Dependent data array must be 1-dimensional, not %d" %  len(oc.shape)
		if m is None:
			m = oc.shape[0]
		assert m == oc.shape[0], "Bad output size: %d vs. previous %d" % (oc.shape[0], m)
		o.append(oc)
	sum = Num.zeros((m,), Num.Float)
	for tmp in o:
		Num.add(sum, tmp, sum)
	avg = sum/len(o)
	sumsq = Num.zeros((m,), Num.Float)
	for tmp in o:
		Num.add(sumsq, (tmp - avg)**2, sumsq)
	return sumsq



def _pack(data, constant=True):
	# Build the data array:
	# Build the array of basis functions:
	ic, oc = data[0][:2]	# We ignore the weight, if any.
	# M will be the number of output coordinates (data points).
	# N will be the number of input coordinates (parameters).
	m = oc.shape[0]
	n = ic.shape[0]
	c = int(constant)
	nd = len(data)
	i = Num.zeros((nd, n+c), Num.Float)
	o = Num.zeros((nd, m), Num.Float)
	for (j, iow) in enumerate(data):
		assert isinstance(iow, tuple)
		if len(iow) == 2:
			w = 1.0
			ic, oc = iow
		elif len(iow) == 3:
			ic, oc, w = iow
			assert w >= 0.0
		else:
			raise ValueError, "Needs 2-tuple (i,o) or 3-tuple (i,o,w)"
		o[j,:] = oc*w
		if constant:
			i[j,0] = 1.0
			i[j,1:] = ic
		else:
			i[j,:] = ic
		Num.multiply(i[j,:], w, i[j,:])
	return (i, o, m, n)




# This is used to compute the matrix of derivitives,
# which is then used to compute the step in mcmcSQ.py
def localfit(data, minsv=None, minsvr=None, constant=True):
	"""Does a linear fit to data via a singular value decomposition
	algorithm.
	It returns the matrix A and vector B such that
	C{A*input_coordinates+B} is the best fit to C{output_coordinates}.
	@param minsv: sets the minimum useable singular value;
	@param minsvr: sets the minimum useable s.v. in terms of the largest s.v.
	@type minsv: float or None
	@type minsvr: float or None
	@param constant: Do you want a constant built into the linear equations?
	@type constant: Boolean
	@return: (A, B, errs, sv, ranks) where
		- A is a 2-D L{numpy.ndarray} matrix.
		- B is a 1-D L{numpy.ndarray} vector.
		- errs is a vector, one value for each output coordinate.
		- sv are the singular values, sorted into decreasing order.
	@param data: [(input_coordinates, output_coordinates), ...].
		A list of (in,out) tuples corresponding to the
		independent and dependent parameters of the linear transform.
		Both C{in} and C{out} are one-dimensional
		L{numpy vectors<numpy.ndarray>}.   They don't need to have
		the same length, though (obviously) all instances of "in"
		need to have the same length and all instances of "out"
		also need to match one another.
	@type data: [(L{numpy.ndarray}, L{numpy.ndarray}), ...]
	"""

	ii, oo, m, n = _pack(data, constant=constant)
	data = None	# Reclaim memory
	soln = gpk_lsq.linear_least_squares(ii, oo, minsv=minsv, minsvr=minsvr)
	ii = None	# Free unneeded memory.
	oo = None	# Free unneeded memory.
	errs = Num.sum( soln.residual()**2, axis=0 )
	sv = soln.sv()
	x = soln.x()
	rank = soln.rank()
	assert sv.shape[0]>=rank and sv.shape[0]<=n+1, "sv.shape=%s rank=%d m=%d n=%d" % (str(sv.shape), rank, m, n)
	assert rank <= 1+n
	assert len(errs) == m
	assert len(x.shape) == 2
	if constant:
		return ( Num.transpose(x[1:,:]), x[0,:],
			errs, Num.sort(sv)[::-1], rank
			)
	return ( Num.transpose(x), None,
			errs, Num.sort(sv)[::-1], rank
			)


# This is used to compute the matrix of derivitives,
# which is then used to compute the step in mcmcSQ.py
def r_localfit(data, minsv=None, minsvr=None):
	"""Data is [ (input_coordinates, output_coordinates), ... ]
	Minsv sets the minimum useable singular value;
	minsvr sets the minimum useable s.v. in terms of the largest s.v..
	It returns the matrix A and vector B such that the best fit
	is A*input_coordinates+B in a tuple
	(A, B, errs, ranks).
	errs is a vector, one value for each output coordinate.
	Rank is the minimum rank of the various fits.
	"""

	ii, oo, m, n = _pack(data)
	data = None	# Reclaim memory
	errs = []
	rank = None
	const = Num.zeros((m,), Num.Float)
	coef = Num.zeros((n,m), Num.Float)
	for j in range(m):
		assert oo.shape[1] == m
		soln = gpk_rlsq.robust_linear_fit(ii, oo[:,j], 1, minsv=minsv, minsvr=minsvr)
		errs.append( Num.sum( soln.residual()**2 ) )
		if rank is None or rank > soln.rank():
			rank = soln.rank()
		const[j] = soln.x()[0]
		coef[:,j] = soln.x()[1:]
	ii = None	# Free unneeded memory.
	oo = None	# Free unneeded memory.
	assert rank <= 1+n
	assert len(errs) == m
	return ( Num.transpose(coef), const, errs, rank)



def fit_giving_sigmas(data, minsv=None, minsvr=None, constant=True):
	"""Does a linear fit to data via a singular value decomposition
	algorithm.
	It returns the matrix A and vector B such that
	C{A*input_coordinates+B} is the best fit to C{output_coordinates}.
	@param minsv: sets the minimum useable singular value;
	@param minsvr: sets the minimum useable s.v. in terms of the largest s.v.
	@type minsv: float or None
	@type minsvr: float or None
	@param constant: Do you want a constant built into the linear equations?
	@type constant: Boolean
	@return: (A, B, errs, sv, ranks) where
		- A is a 2-D L{numpy.ndarray} matrix.
		- B is a 1-D L{numpy.ndarray} vector.
		- errs is a vector, one value for each output coordinate.
		- sv are the singular values, sorted into decreasing order.
	@param data: [(input_coordinates, output_coordinates), ...].
		A list of (in,out) tuples corresponding to the
		independent and dependent parameters of the linear transform.
		Both C{in} and C{out} are one-dimensional
		L{numpy vectors<numpy.ndarray>}.   They don't need to have
		the same length, though (obviously) all instances of "in"
		need to have the same length and all instances of "out"
		also need to match one another.
	@type data: [(L{numpy.ndarray}, L{numpy.ndarray}), ...]
	"""

	ii, oo, m, n = _pack(data, constant=constant)
	data = None	# Reclaim memory
	soln = gpk_lsq.linear_least_squares(ii, oo, minsv=minsv, minsvr=minsvr)
	ii = None	# Free unneeded memory.
	oo = None	# Free unneeded memory.
	x = soln.x()
	sigmas = Num.sqrt(soln.x_variances())
	assert sigmas.shape == x.shape
	assert len(x.shape) == 2
	if constant:
		return ( Num.transpose(x[1:,:]), x[0,:],
			 Num.transpose(sigmas[1:,:]), sigmas[0,:]
			)
	return ( Num.transpose(x), None,
			Num.transpose(sigmas), None
			)


def leaktest():
	import RandomArray
	while 1:
		d = []
		for i in range(100):
			d.append( (RandomArray.standard_normal((30,)),
					RandomArray.standard_normal((1000,))))
		localfit(d)

def test_localfit11(r):
	d = [	(Num.array((0,)), Num.array((-1,))),
		(Num.array((1,)), Num.array((0,))),
		(Num.array((2,)), Num.array((1+1e-7,))),
		(Num.array((3,)), Num.array((2,))),
		]
	if r:
		coef, const, errs, sv, rank = r_localfit(d)
	else:
		coef, const, errs, sv, rank = localfit(d)
	assert const.shape == (1,)
	assert abs(const[0]-(-1)) < 1e-6
	assert coef.shape == (1,1)
	assert abs(coef[0,0]-1) < 1e-6
	assert len(errs)==1
	assert errs[0] < 1e-6
	assert len(sv) == 2
	assert rank == 2
	assert Num.alltrue(err_before_fit(d) >= errs)


def test_localfit11e():
	d = [	(Num.array((0.0,)), Num.array((0.0,))),
		(Num.array((1.0,)), Num.array((1.0,))),
		(Num.array((2.0,)), Num.array((0.0,))),
		(Num.array((3.0,)), Num.array((-2.0,))),
		(Num.array((4.0,)), Num.array((0.0,))),
		(Num.array((5.0,)), Num.array((1.0,))),
		(Num.array((6.0,)), Num.array((0.0,))),
		]
	coef, const, errs, sv, rank = localfit(d)
	print 'coef=', coef
	print 'const=', const
	print 'errs=', errs
	assert const.shape == (1,)
	assert abs(const[0]-0) < 1e-6
	assert coef.shape == (1,1)
	assert abs(coef[0,0]-0) < 1e-6
	assert len(errs)==1
	assert abs(errs[0]-6) < 1e-6
	assert len(sv) == 2
	assert rank == 2
	assert Num.sum(Num.absolute(err_before_fit(d)-errs)) < 1e-6


def test_localfit21():
	d = [ (Num.array((0,0)), Num.array((-1,))),
		(Num.array((1,2)), Num.array((0,))),
		(Num.array((2,-1)), Num.array((1,))) ]
	coef, const, errs, sv, rank = localfit(d)
	assert const.shape == (1,)
	assert abs(const[0]-(-1)) < 1e-6
	assert coef.shape == (1,2)
	assert Num.sum(Num.absolute(coef-[[1,0]])) < 1e-6
	assert len(errs)==1
	assert errs[0] < 1e-6
	assert len(sv) == 3
	assert rank == 3
	assert Num.alltrue(err_before_fit(d) >= errs)

def test_localfit21u():
	d = [ (Num.array((0,0)), Num.array((-1,))),
		(Num.array((2,-1)), Num.array((1,))) ]
	coef, const, errs, sv, rank = localfit(d)
	assert const.shape == (1,)
	assert abs(const[0]-(-1)) < 1e-6
	assert abs(const[0] + 2*coef[0,0]-coef[0,1] - 1) < 1e-6
	assert coef.shape == (1,2)
	assert len(errs)==1
	assert errs[0] < 1e-6
	assert len(sv) == 2
	assert rank == 2
	assert Num.alltrue(err_before_fit(d) >= errs)

def test_localfit22(r):
	d = [ (Num.array((0,0)), Num.array((-1,0))),
		(Num.array((1,2)), Num.array((0,1.0))),
		(Num.array((-1,2)), Num.array((-2,1.0))),
		(Num.array((-1,1)), Num.array((-2+1e-7,0.5+1e-7))),
		(Num.array((2,-1)), Num.array((1,-0.5))) ]

	if r:
		coef, const, errs, sv, rank = r_localfit(d)
	else:
		coef, const, errs, sv, rank = localfit(d)

	print const
	print coef
	print  errs
	print  sv
	print rank
	assert const.shape == (2,)
	assert Num.sum(Num.absolute(const - [-1,0])) < 1e-6
	assert coef.shape == (2,2)
	assert Num.sum(Num.absolute(coef[0]-[1,0])) < 1e-6
	assert Num.sum(Num.absolute(coef[1]-[0,0.5])) < 1e-6
	assert len(errs)==2
	assert Num.sum(errs) < 1e-6
	assert len(sv) == 3
	assert rank == 3
	assert Num.alltrue(err_before_fit(d) >= errs)


def test_fgs1():
	N = 2000
	DSIG = 10.0
	import random
	import math
	d = []
	for i in range(N):
		s = random.normalvariate(0.0, DSIG)
		d.append((Num.array([-1.0]), Num.array([1-s])))
		d.append((Num.array([1.0]), Num.array([1+s])))
	coef, const, scoef, sconst = fit_giving_sigmas(d)
	assert abs(coef[0][0]) < 3*DSIG/math.sqrt(N)
	assert abs(coef[0][0]) > 0.001*DSIG/math.sqrt(N)
	assert abs(const[0]-1.0) < DSIG/math.sqrt(N)
	assert sconst[0] < DSIG/math.sqrt(N)
	assert abs(coef[0][0]) < 4*DSIG/2.0/math.sqrt(N)
	assert scoef[0][0] < 1.3*DSIG/math.sqrt(N)
	assert scoef[0][0] > 0.7*DSIG/math.sqrt(N)
	print 'FGS1 OK'



if __name__ == '__main__':
	test_localfit11(False)
	test_localfit11e()
	test_localfit21()
	test_localfit21u()
	for r in [False, True]:
		test_localfit11(r)
		test_localfit22(r)
	test_fgs1()



"""A multithreaded version of os.popen2().  Note that the argument list
isn't quite the same."""


import os




class pfd(object):
	__doc__ = """Pseudo-file descriptor.  Just like a FD, except
		that it waits for the process at the end of the pipe
		to terminate when you close it.
		"""

	def __init__(self, fd, cpid):
		self.fd = os.fdopen(fd, "r")
		self.cpid = cpid
		self.mode = "r"
		self.waitval = None
		self.closed = False

		self.readline = self.fd.readline
		self.next = self.fd.next
		self.flush = self.fd.flush
		self.__iter__ = self.fd.__iter__
		self.read = self.fd.read
		self.readlines = self.fd.readlines
		self.fileno = self.fd.fileno

	# def read ... copied from file.read
	def read(self):
		return self.fd.read()

	# def readline ... copied from file.readline
	def readline(self):
		return self.fd.readline()

	# def readlines ... copied from file.readlines
	def readlines(self):
		return self.fd.readlines()

	# def next ... copied from file.next
	def next(self):
		return self.fd.next()

	# def __iter__ ... copied from file.__iter__

	def __iter__(self):
		return self.fd.__iter__()

	# def flush ... copied from file.flush
	def flush():
		return

	# def fileno ... copied from file.fileno


	def xreadlines(self, sizehint=-1):
		return self.fd.xreadlines(sizehint)


	def close(self):
		if not self.closed:
			self.fd.close()
			self.closed = True
			pid, t2 = os.waitpid(self.cpid, 0)
			if t2 == 0:
				self.waitval = None
			else:
				self.waitval = t2
		return self.waitval


	def __del__(self):
		self.close()


def popen2(path, args):
	"""Forks off a process, and returns the processes
	input and output file descriptors.  The latter is really
	a pfd (defined above).

	Path and args are passed directly into os.execvp().
	"""

	rci, wci = os.pipe()
	rco, wco = os.pipe()

	cpid = os.fork()
	if cpid == 0:
		# child
		os.close(rco)
		os.close(wci)
		os.dup2(rci, 0)
		os.dup2(wco, 1)
		os.execvp(path, args)
		os._exit(127)

	os.close(rci)
	os.close(wco)
	return (os.fdopen(wci, "w"), pfd(rco, cpid))


def test():
	si, so = popen2("sed", ["sed", "-e", "s/or/er/"])
	si.write("hello, world!\nsecond line\n")
	si.close()
	assert so.readline() == "hello, werld!\n"
	a = so.readlines()
	assert len(a) == 1
	assert a[0] == "second line\n"
	tmp = so.close()
	assert tmp is None

if __name__ == '__main__':
	test()
	test()



"""Fit a plane to some data."""


import g_lin_fit
import Num


class Error(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)


class NoDataError(ValueError):
	def __init__(self, s):
		ValueError.__init__(self, s)

def plane(data, wt=None):
	"""Fit a plane to some data.
	where the fitting function is
	f = c[0]*1 + c[1]*dep[0] + c[2]*dep[1] + ...
	where c is the array of coefficients.
	The length of c is equal to the number of dependent
	variables in each datum.
	@param data: [ (independent, dependent, dependent...), ...]
	@return: (c, opt, resid, rank, sv), where
		bestfit is the best fit to the data
		(i.e., the values of f),
		resid is the residual (a single float number),
		rank the rank of the fit (int),
		and sv is the array of the singular values.
	"""
	idata = [ ( [ t[0] for t in data ], None) ]
	n = len(data)
	if n <= 0:
		raise NoDataError('No Data')

	dims = len(data[0])
	for t in data:
		if len(t) != dims:
			raise ValueError('Data length mismatch')
	ffi = Num.zeros((n, dims), Num.Float)
	ffi[:, 0] = 1
	for i in range(1, dims):
		for j in range(n):
			ffi[j, i] = data[j][i]

	ff = [lambda info, ffin, nn, ii=i: ffin[:,ii] for i in range(dims) ]

	opt, resid, rank, sv, bestfit = g_lin_fit.fit(idata, ff, ffi, wt)
	return (opt, bestfit[0], resid, rank, sv)



def test():
	data = [ (1, 0), (2, 2), (3, 4), (4, 6) ]
	opt, bestfit, resid, rank, sv = plane(data)
	assert rank == 2
	assert abs(resid) < 1e-6
	for i in range(len(data)):
		assert abs(bestfit[i] - data[i][0]) < 1e-6
	assert abs(opt[0] - 1) < 1e-6 and abs(opt[1]-0.5) < 1e-6

if __name__ == '__main__':
	test()




import os

def _fmtglobal(d):
	o = []
	for (k, v) in d.items():
		o.append('%s=%s' % (str(k), str(v)) )
	return ' '.join(o)




class writer:
	def __init__(self, fd):
		self.globals = []
		self.regions = []
		self.fd = fd
	

	def header(self, d):
		self.globals.append( 'global ' + _fmtglobal(d) )
	
	def text(self, x, y, text):
		self.regions.append('image;text(%g,%g) # text={%s}'
					% (x, y, text)
					)
	
	def close(self):
		for g in self.globals:
			self.fd.writelines(g + '\n')
		for r in self.regions:
			self.fd.writelines(r + '\n')
		self.fd.flush()
		os.fsync(self.fd.fileno())
		self.fd = None

	
	def __del__(self):
		if self.fd is not None:
			self.close()



import math
from gmisclib import die
from gmisclib import avio
try:
	from gmisclib import load_mod
except ImportError:
	pass

"""This lets you select lines from a list of dictionaries
by means of a little snippet of python code.
It can be used to select lines in a fiat or avio format file
(as read in by fiatio.read or avio.read).
"""

def filterlist(s, d, verbose=False):
	"""This filters a list of dictionaries, passing ones where
	the X{selector} returns true.
	@param s: the selector -- a little snippet of python
	@type s: L{str}
	@param d: the list of dictionaries.
		Each dictionary is temporarily used as the local variable
		space and the selector is evaluated.
	@type d: C{ [ { "key": value, ... }, ...] }
	@return: a subset of the input list, whichever dictionaries cause
		C{s} to return True when evaluated.
	@rtype C{ [ { "key": value, ...}, ...] }
	"""
	return list(filter_iter(s, d, verbose=verbose))


def filter_iter(s, d, verbose=False):
	"""This filters a list of lines, passing ones where
	the selector returns true.
	S is the selector -- a little snippet of python,
	and d is the list of data : a list of {k:v} mappings.
	"""
	nok = 0
	nne = 0
	x = None
	cs = _compile(s)
	for t in d:
		if verbose:
			die.info('filter_iter: %s' % str(t))
		try:
			if cs.eval(t):
				nok += 1
				yield(t)
		except NameError, x:
			nne += 1
	if nne>0 and nok==0:
		die.warn('Name Error (%s) on each of the %d data.' % (str(x), nne))


def _compact(s):
	if len(s) > 15:
		return s[:12] + "..."
	return s


class selector_c(object):
	def __init__(self, code, global_values=None):
		"""Code = a string containing python code.
		Global_values = a dictionary containing values to be used by
		that python code.  Note that the dictionary is not copied,
		so that it is shared and changes will be noticed.
		"""
		self.set_code(code)
		if global_values is None:
			self.g = {'math': math}
			try:
				self.g['load_mod'] = load_mod
			except NameError:
				pass
		else:
			self.g = global_values
		self.traptuple = ()
		self.trapmap = []

	def set_code(self, code):
		self.cc = compile(code, '<g2_select: %s>' % _compact(code), 'eval')

	def set_trap(self, exc, result):
		self.traptuple = self.traptuple + (exc,)
		self.trapmap.append( (exc, result) )

	def eval(self, locals):
		try:
			return eval(self.cc, self.g, locals)
		except self.traptuple, x:
			for (e,r) in self.trapmap:
				if isinstance(x, e):
					return r
			raise

	def globals(self, s):
		exec s in self.g


CCSZ = 100
_ccache = {}
def _compile(s):
	# print 'Compile', s
	try:
		selector = _ccache[s]
	except KeyError:
		if len(_ccache) > CCSZ:
			_ccache.pop()
		_ccache[s] = selector_c(s)
		selector = _ccache[s]
	return selector


def accept(s, d):
	"""This checks a single dictionary, and returns
	the result of the selector.   Errors in the evaluation
	are trapped and cause accept() to return False.
	S is the selector -- a little snippet of python,
	and d is the data : a {k:v} mapping.
	"""
	try:
		return evaluate(s, d)
	except NameError, x:
		die.warn('Name Error: %s on %s in %s' % (str(x), avio.concoct(d), s))
	return False


def whynot(s, d):
	"""Returns an explanation of why d was not accepted, given s, or None of d was accepted."""
	try:
		if _compile(s).eval(d):
			return None
		else:
			return "Sorry, not implemented yet."
	except NameError, x:
		return "data does not contain attribute=%s (d=%s) (selector=%s)" % (x.args[0],
								_compact(avio.concoct(d)),
								_compact(s)
								)

def why(s, d):
	"""Returns an explanation of why d was accepted, given s, or None if d was not accepted."""
	if not accept(s, d):
		return None
	return "Sorry, not implemented yet."


def evaluate(s, d):
	"""This checks a single dictionary, and returns
	the result of the selector.
	S is the selector -- a little snippet of python,
	and d is the data : a {k:v} mapping.
	"""
	return _compile(s).eval(d)


def test():
	x = selector_c('y')
	assert x.eval( {'y': 1} ) == 1
	try:
		x.eval( {} )
	except NameError:
		pass
	else:
		assert 0, 'Whoops! no exception when one was expected.'
	z = selector_c('y+w', {'w':1})
	assert z.eval( {'y': 1} ) == 2
	z.globals('w=2')
	assert z.eval( {'y': 1} ) == 3
	x = selector_c('y')
	x.trap(NameError, 44)
	assert x.eval({}) == 44



if __name__ == '__main__':
	test()



"""V=WH, where W and H are non-negative.

From D. D. Lee and H. S. Seung, 'Learning the parts of objects by
nonnegative matrix factorization.'
"""

import Num
import math
import die



EPS = 1e-6
CC = 2
IEPS = 0.03

def _norm(x):
	try:
		tmp = math.sqrt(Num.sum(x**2, axis=None))
	except OverflowError, x:
		print "x=", x
		raise
	return tmp
	

def _converged(wh, whold, v, fudge):
	F = math.sqrt(wh.shape[0] * wh.shape[1])
	d1 = _norm(wh-whold)/_norm(v)
	d2 = _norm(v-wh)/_norm(v)
	die.dbg("Converged= %f %f" % (d1, d2))
	return min(d1*F,d2) < EPS*fudge


def _updateH(w, h, wh, v):
	# eps = EPS*(Num.sum(v)
	eps = EPS * Num.sum(wh, axis=None)/(v.shape[0]*v.shape[1])
	# f = Num.matrixmultiply(Num.transpose(w), (v+eps)/(wh+eps))
	f = Num.matrixmultiply(Num.transpose(w), v/(wh+eps))
	# print "H: f=", f
	return h * f


def _updateW(w, h, wh, v):
	eps = EPS * Num.sum(wh, axis=None)/(v.shape[0]*v.shape[1])
	print 'eps=', eps
	# f = Num.matrixmultiply((v+eps)/(wh+eps), Num.transpose(h))
	f = Num.matrixmultiply(v/(wh+eps), Num.transpose(h))
	print 'f=', f
	print "W:v/wh=", v/(wh+eps)
	print "W:f=", f
	new_w = w * f
	print "new_w=", new_w
	first_index_sum = Num.sum(new_w, axis=0)
	print "fis=", first_index_sum
	print 'new_w=', new_w
	o = new_w/first_index_sum[Num.NewAxis, :]
	print "o=", o
	return o


def _initialize(v, rank):
	n, m = v.shape
	w = Num.RA.standard_normal((n, rank))**2 + IEPS
	c = Num.sum(v**2, axis=0)/(v.shape[0]*v.shape[1])
	h = c * (Num.RA.standard_normal((rank, m))**2 + IEPS)
	return (w, h)


def nmf(v, rank):
	assert rank > 0, "Zero rank approximations are usually pretty awful."
	v = Num.asarray(v, Num.Float)
	assert Num.alltrue(Num.greater_equal(Num.ravel(v), 0.0)), "Negative element!"
	w, h = _initialize(v, rank)
	cc = 0
	ic = 0
	wh = Num.zeros(v.shape, Num.Float)
	while 1:
		# die.dbg("Loop, cc= %d" % cc)
		whold = wh
		wh = Num.matrixmultiply(w, h)
		if _converged(wh, whold, v, math.sqrt(ic/float(rank))):
			cc += 1
		else:
			cc = 0
		# print "cc=", cc
		if cc > CC:
			break
		wnew = _updateW(w, h, wh, v)
		hnew = _updateH(w, h, wh, v)
		w = wnew
		h = hnew
		ic += 1

	return (w, h, _norm(v - Num.matrixmultiply(w, h)) )




def _test1():
	a = [[1, 0], [1, 0], [0, 0]]
	w, h, err = nmf(a, 1)
	wh = Num.matrixmultiply(w, h)
	assert _norm(wh - a) < 30*EPS
	assert err<0.001

def _test2():
	a = [[1, 0], [0, 1], [0, 0]]
	w, h, err = nmf(a, 2)
	wh = Num.matrixmultiply(w, h)
	assert _norm(wh - a) < 30*EPS
	assert err<0.001


def _test3( rank ):
	a = [[1.0, 0.0, 0.5], [0.0, 1.0, 0.5], [1.0, 1.0, 1.0]]
	w, h, err = nmf(a, rank)
	wh = Num.matrixmultiply(w, h)
	assert _norm(wh - a) < 30*math.sqrt(EPS)
	assert err<0.001


if __name__ == '__main__':
	print "TEST1"
	_test1()
	print
	print "TEST2"
	_test2()
	print
	print "TEST3(1)"
	try:
		_test3(1)
	except AssertionError:
		pass
	else:
		raise AssertionError, "Test3(1) should fail!"
	print
	print "TEST3(2)"
	_test3(2)
	print
	print "TEST3(3)"
	_test3(3)
	print
	print "TEST3(4)"
	_test3(4)
	# print
	print "TEST3(5)"
	_test3(5)
	print
	print "LAST"

	a = [[1, 0], [0, 1]]
	w, h, err = nmf(a, 4)
	print "w=", w
	print "h=", h
	print "wh=", Num.matrixmultiply(w, h)
	print "a=", a



import re
import die

"""This selects dictionaries (i.e. sets of attribute=value pairs),
to see if they meet specified criteria.
"""


class selector:
	def accept(self, item):
		return self.x.accept(item)

	def __init__(self):
		self.x = true()

	def and_another(self, txt):
		if isinstance(self.x, true):
			self.x = one_selector(txt)	# Leaf node
		elif isinstance(self.x, false):
			pass
		else:
			self.x = selector_op(self.x, _and, one_selector(txt) )

	select = and_another

	def or_another(self, txt):
		if isinstance(self.x, false):
			self.x = one_selector(txt)	# Leaf node
		elif isinstance(self.x, true):
			pass
		else:
			self.x = selector_op(self.x, _or, one_selector(txt) )
	
	def And(self, s):
		self.x = selector_op(self.x, _and, s)
	
	def Or(self, s):
		self.x = selector_op(self.x, _or, s)

	def Not(self):
		self.x = selector_not(self.x)
	
	def leaf(self, txt):
		self.x = one_selector(txt)

	def filterlist(self, x):
		o = []
		for datum in x:
			if self.accept(datum):
				o.append( datum )
		return o


class true(selector):
	def __init__(self):
		pass
	def accept(self, item):
		return True

class false(selector):
	def __init__(self):
		pass
	def accept(self, item):
		return False


def _and(x, l, r):
	if l.accept(x):
		return r.accept(x)
	return False


def _or(x, l, r):
	if l.accept(x):
		return True
	return r.accept(x)


class selector_op(selector):
	def __init__(self, l, op, r):
		self.l = l
		self.r = r
		self.op = op

	def accept(self, x):
		return (self.op)(x, self.l, self.r)


def selector_not(selector):
	def __init__(self, l):
		self.l = l

	def accept(self, x):
		return not self.l(x)



class one_selector(selector):
	_eq = re.compile('^(.*)[.]eq[.](.*)$')
	_eqn = re.compile('^(.*)==(.*)$')
	_ne = re.compile('^(.*)[.]ne[.](.*)$')
	_nen = re.compile('^(.*)!=(.*)$')
	_ew = re.compile('^(.*)[.]endswith[.](.*)$')
	_sw = re.compile('^(.*)[.]startswith[.](.*)$')
	_ex = re.compile('^(.*)[.]exists[.]?$')
	def __init__(self, txt):
		if self._eq.match(txt):
			m = self._eq.match(txt)
			self.op = lambda t, k=m.group(1), v=m.group(2): t[k]==v
		elif self._ne.match(txt):
			m = self._ne.match(txt)
			self.op = lambda t, k=m.group(1), v=m.group(2): t[k]!=v
		elif self._eqn.match(txt):
			m = self._eqn.match(txt)
			self.op = lambda t, k=m.group(1), v=float(m.group(2)): float(t[k])==float(v)
		elif self._nen.match(txt):
			m = self._nen.match(txt)
			self.op = lambda t, k=m.group(1), v=float(m.group(2)): float(t[k])!=float(v)
		elif self._ew.match(txt):
			m = self._ew.match(txt)
			self.op = lambda t, k=m.group(1), v=m.group(2): t[k].endswith(v)
		elif self._sw.match(txt):
			m = self._sw.match(txt)
			self.op = lambda t, k=m.group(1), v=m.group(2): t[k].startswith(v)
		elif self._ex.match(txt):
			m = self._ex.match(txt)
			self.op = lambda t, k=m.group(1): t.has_key(k)
		else:
			die.die("Unrecognized operator: %s" % txt)

	def accept(self, item):
		return (self.op)(item)



def test():
	s = selector()
	s.select('a.eq.3')
	assert s.accept({'a': '3'})
	assert not s.accept({'a': '4'})
	s = selector()
	s.select('a.eq.3')
	s.select('b.ne.5')
	assert s.accept({'a': '3', 'b': '4'})
	assert s.accept({'a': '3', 'b': 'k'})
	assert not s.accept({'a': '3', 'b': '5'})
	assert not s.accept({'a': '2', 'b': '6'})
	assert not s.accept({'a': '2', 'b': '5'})
	s.or_another('a.eq.2')
	assert s.accept({'a': '2', 'b': '5'})
	s = selector()
	s.select('a.endswith.2')
	assert s.accept({'a': '2342'})



if __name__ == '__main__':
	test()



"""These are I/O routines to allow you to write stuff
like arrays and dictionaries (and arrays of dictionaries)
to a human-readable file.

The format is normally STARTMARKER LENGTH_INFO DATA ENDMARKER,
where STARTMARKER is something like C{"a{"},
that specifies you're at the beginning of an array.
LENGTH_INFO can depend on the data type, but it's normally
an integer.   Then comes the data, and finally as a check,
you encounter the ENDMARKER, which is normally C{"}"}.
All of this is recursive, of course.
"""


import os
import string
import collections
from gmisclib import Num
from gmisclib import g_encode

_e = g_encode.encoder(string.letters + string.digits + r"""<>?,./:";'{}[\]!@$^&*()_+=\|\\-""")
__version__ = "$Revision: 1.24 $"

def test_e():
	assert _e.fwd(' ') == '%S'
	assert _e.fwd('hello there') == 'hello%Sthere'
	assert not _e.fwd('#x').startswith('#')


class BadFileFormat(RuntimeError):
	def __init__(self, s):
		RuntimeError.__init__(self, s)

class chunk:
	__doc__ = """Low level file I/O operations.
		This class represents a sequence
		of white-space separated chunks of data.
		"""

	def __init__(self):
		"""Constructor."""
		pass


	def more(self):
		"""Returns zero if the data source is empty.
		Returns nonzero if there is more data.
		"""
		raise RuntimeError, 'Virtual Function'


	def readchunk(self):
		"""Read in the next white-space delimited chunk of text."""
		raise RuntimeError, 'Virtual Function'


	def read_float(self):
		tmp = self.readchunk()
		if tmp != 'f:':
			raise BadFileFormat, 'read_float: bad prefix'
		return float(self.readchunk())


	def read_array(self, fcn):
		"""Read an array of data values.  Raw text is converted to
		finished array values by the specified fcn."""
		
		return self.read_array_of( lambda s: fcn(s.readchunk()) )

	def read_tuple(self, fcn):
		"""Read a tuple of data values.  Raw text is converted to
		finished array values by the specified fcn."""
		
		return tuple( self.read_array_of( lambda s: fcn(s.readchunk()) ) )


	_array_prefix = 'a{'
	_lap = len(_array_prefix)

	def read_array_of(self, fcn):
		"""Read an array of data values.  Values are read
		in by the specified fcn."""
		
		tmp = self.readchunk()
		if tmp is None:
			return None
		if not tmp.startswith(self._array_prefix):
			raise BadFileFormat, 'Array initial'
		if len(tmp) > self._lap:
			n = int(tmp[self._lap:])
		else:
			n = int(self.readchunk())
		o = [ fcn(self) for i in range(n) ]
		if self.readchunk() != '}':
			raise BadFileFormat, 'Array final'
		return o


	def read_dict(self, fcn):
		"""Read a dictionary of data values.  Raw text is converted to
		finished values by the specified fcn. Keys are strings."""

		return self.read_dict_of( lambda self, ff=fcn: ff(self.readchunk()) )

	_dict_prefix = 'd{'
	_ldp = len(_dict_prefix)

	def read_dict_of(self, fcn):
		"""Read a dictionary of data values.
		Values are read in by the specified fcn.
		This allows dictionaries of X, where X can be a complex datatype
		like an array.  Keys are strings."""

		tmp = self.readchunk()
		if tmp is None:
			return None
		if not tmp.startswith('d{'):
			raise BadFileFormat, 'Dict initial'
		if len(tmp) > self._ldp:
			n = int(tmp[self._ldp:])
		else:
			n = int(self.readchunk())

		o = {}
		for i in range(n):
			k = self.readchunk()
			v = fcn(self)
			o[k] = v
		if self.readchunk() != '}':
			raise BadFileFormat, 'Dict final'
		return o


	def groupstart(self):
		tmp = self.readchunk()
		if not tmp.startswith('g{'):
			raise BadFileFormat, 'Group initial: "%s"' % tmp
		return self.readchunk()


	def groupend(self):
		if self.readchunk() != '}':
			raise BadFileFormat, 'Group final'


	def read_NumArray(self):
		tmp = self.readchunk()
		if tmp != 'N{':
			raise BadFileFormat, 'NumArray initial: "%s"' % tmp
		sz = tuple(self.read_array(int))
		n = 1
		for s in sz:
			assert s > 0  and  s < 100000
			n *= s
		try:
                	if len(sz) == 2:
				d = Num.zeros(sz, Num.Float)
                        	for i in range(sz[0]):
                                	for j in range(sz[1]):
						d[i, j] = float(self.readchunk())
                	else:
				d = Num.zeros((n,), Num.Float)
                        	for i in range(n):
					d[i] = float(self.readchunk())
				d = Num.reshape(d, sz)
		except ValueError, x:
			if str(x).startswith('invalid literal for float'):
				raise BadFileFormat, 'Expected floats, got something else: %s' % str(x)
		assert d.shape == sz
		if self.readchunk() != '}':
			raise BadFileFormat, 'NumArray final'
		return d


class datachunk(chunk):
	__doc__ = """Low level file I/O operations.
		This class represents a file as a sequence
		of white-space separated chunks of data.
		"""

	def __init__(self, fd):
		"""Constructor."""
		chunk.__init__(self)
		# self.readiter = gpkmisc.threaded_readable_file(fd)
		self.readiter = fd
		self.next = collections.deque( self._get_next() )


	def _get_next(self):
		while True:
			nxt = self.readiter.readline()
			if nxt == '':
				return []
			if nxt.startswith('#'):
				continue
			nxt = nxt.strip()
			if nxt == '':
				continue
			return nxt.split()


	def more(self):
		"""Returns False if the data source is empty.
		@return: True if there is more data.
		"""
		if len(self.next):
			return True
		self.next.extend( self._get_next() )
		return len(self.next) > 0


	def readchunk(self):
		"""Read in the next white-space delimited chunk of text."""
		try:
			return _e.back( self.next.popleft() )
		except IndexError:
			self.next.extend( self._get_next() )
			if len(self.next) == 0:
				return None
		return _e.back( self.next.popleft() )


class stringchunk(chunk):
	__doc__ = """Low level operations: splitting a string into chunks.
		This class represents a string as a sequence
		of white-space separated chunks of data.
		"""

	def __init__(self, s):
		"""Constructor."""
		chunk.__init__(self)
		self.cache = s.split()
		self.i = 0


	def more(self):
		"""@return: zero if the data source is empty; nonzero if there is more data.
		"""
		return self.i < len(self.cache)-1


	def readchunk(self):
		"""Read in the next white-space delimited chunk of text."""
		try:
			tmp = _e.back(self.cache[self.i])
			self.i += 1
			return tmp
		except IndexError:
			return None



class chunk_w:
	def __init__(self):
		pass


	def _indent(self):
		pass


	def _dedent(self):
		pass


	def writechunk(self, ch, b=0):
		self.stringwrite(ch, b)
		return self


	def write(self, ch, b=0):
		raise RuntimeError, "Virtual function"


	def nl(self):
		raise RuntimeError, "Virtual function"


	def comment(self, comment):
		raise RuntimeError, "Virtual function"


	def close(self):
		raise RuntimeError, "Virtual function"


	def write_None(self):
		self.stringwrite("N")
		return self


	def write_array_of(self, data, writer, b=0):
		self.stringwrite("a{%d" % len(data), b)
		self._indent()
		for t in data:
			writer(self, t)
		self._dedent()
		self.stringwrite("}", b)
		return self


	def write_array(self, data, b=0, converter=str):
		return self.write_array_of(data,
				lambda s, x, cvt=converter: s.stringwrite(cvt(x)),
				b)


	def write_dict_of(self, data, writer, b=1):
		self.stringwrite("d{%d" % len(data), b)
		self._indent()
		for (k, v) in data.items():
			self.stringwrite(k)
			writer(self, v)
		self._dedent()
		self.stringwrite("}", b)
		return self


	def write_dict(self, data, b=0, converter=str):
		return self.write_dict_of(data,
				lambda s, v, c=converter: s.stringwrite(c(v)),
				b)
	

	def groupstart(self, contents, b=1, comment=None):
		"""The 'contents' argument is conventionally a string
		containing 'a' for an internal array, 'g' for a group,
		'd' for a dictionary...  It describes the contents of the group.
		This is just used by the application to check what is in the
		group, so it could contain any chunk.
		"""
		self.stringwrite("g{", b)
		self._indent()
		self.stringwrite(contents)
		if comment:
			self.comment(comment)
		return self


	def groupend(self, b=1):
		self.stringwrite("}", b)
		self._dedent()
		return self


        def write_NumArray(self, d, b=0):
                self.stringwrite("N{", b)
                self.write_array(d.shape, b=0)
                if len(d.shape) == 2:
                        for i in range(d.shape[0]):
                                for j in range(d.shape[1]):
                                        self.stringwrite( str(d[i, j]) )
				if d.shape[0] > 1:
                                	self.nl()
                else:
                        for t in Num.ravel(d):
                                self.stringwrite( str(t) )
                self.stringwrite("}", 0)
		return self
                                                                                            
	def write_float(self, d, b=0):
		self.stringwrite("f:", b)
		self.stringwrite(str(d), b=0)
		return self

	def stringwrite(self, ch, b=0):
		raise RuntimeError, 'Virtual Function'




class datachunk_w(chunk_w):
	__doc__ = """This writes stuff to a file."""


	def __init__(self, fd, width=80):
		chunk_w.__init__(self)
		self.fd = fd
		self.w = width
		self.i = 0
		self.indentlevel = 0
		self.dentstring = ' '


	def _indent(self):
		self.indentlevel += 1


	def _dedent(self):
		assert self.indentlevel > 0
		self.indentlevel -= 1


	def stringwrite(self, ch, b=0):
		"""ch is the chunk of text.  b=1 to begin a new line."""
		assert isinstance(ch, str), "Non-string to stringwrite: <%s>" % str(ch)
		n = len(ch)
		if self.i == 0:
			pass
		elif b or (self.i > 0  and  n+self.i > self.w):
			self.nl()
		else:
			self.fd.write(' ')
			self.i += 1
		if self.i == 0:
			self.fd.write(self.dentstring * self.indentlevel)
		tmp = _e.fwd(ch)
		self.fd.write(tmp)
		self.i += len(tmp)


	def nl(self):
		self.fd.write('\n')
		self.i = 0


	def comment(self, comment):
		if self.i > 0:
			self.nl()
		idl = max(0, self.indentlevel-1)
		self.fd.write('#' + self.dentstring * idl + comment + '\n')


	def close(self):
		if self.i > 0:
			self.nl()
		self.fd.flush()
		os.fsync(self.fd.fileno())
		self.fd = None	# This will normally close the fd,
				# unless it is held open by some other
				# use.


	def __del__(self):
		if self.fd is not None:
			self.close()




class chunkstring_w(chunk_w):
	__doc__ = """This accumulates stuff in memory, and returns a string
		when close() is called."""

	def __init__(self):
		chunk_w.__init__(self)
		self.buf = []


	def stringwrite(self, ch, b=0):
		self.buf.append(_e.fwd(ch))


	def nl(self):
		pass


	def comment(self, comment):
		pass


	def close(self):
		o = ' '.join(self.buf)
		self.buf = []
		return o



def test():
# print chunkstring_w().write_array([1.3, 2.2], str).close()
	assert stringchunk(chunkstring_w().writechunk('Hoo!#\n').close()).readchunk() == 'Hoo!#\n'
	tmp = Num.array([[1.0, -1.0], [1.6, 0.0], [2.0, 3.0]], Num.Float)
	tmp1 = chunkstring_w().write_NumArray(tmp).close()
	retmp = stringchunk(tmp1).read_NumArray()
	assert retmp.shape == tmp.shape
	diff = retmp - tmp
	assert Num.sum(Num.absolute(Num.ravel(diff))) < 1e-6


if __name__ == '__main__':
	test_e()
	test()



"""Take a covariance-like matrix, and re-order it to be close to a diagonal matrix.
We calculate a map that renames the indices (the same map for both left and right),
to bring large off-diagonal elements close to the diagonal.
"""

import random
import Num
import math


def _eval_map(a, m, wt):
	"""This is a measure of how far the matrix is from diagonal,
	after application of the mapping.
	Matrix "a" is a numeric python 2-d array.
	Map is a numeric python 1-d integer array."""

	n = a.shape[0]
	assert a.shape == (n, n)
	# print "mapped a=", map_array(a, m)
	# print "m=", m
	return Num.sum(map_array(a, m)*wt)


def _offdiag(n):
	index = Num.arrayrange(n)
	offdiag = Num.absolute(index[:,Num.NewAxis]-index[Num.NewAxis,:])
	assert offdiag.shape == (n, n)
	assert offdiag[0, 0] == 0
	assert offdiag[1, 0] == 1
	# print "WT=", offdiag
	return offdiag



def _optimistic(n):
	return Num.ones((n, n), Num.Int) - Num.identity(n)



def diagonalize(a0):
	EPS = 1e-10
	a = Num.array(a0, copy=True)
	n = a.shape[0]
	assert a.shape == (n, n)
	wt = _offdiag(n)**2
	m = Num.arrayrange(n)
	opval = _eval_map(a, m, _optimistic(n))
	# print "OPVAL=", opval
	bad = _eval_map(a, m, wt)
	bad0 = bad
	f = 1.0
	fr = math.pow(0.5, 1.0/n)
	nobetter = 0
	while 1:
		# i = random.choice(_worst(a, m, wt))
		i = random.randrange(n)
		j = random.randrange(n)
		if j == i:
			continue
		# print "TESTING", i, j
		tmp = m[i]
		m[i] = m[j]
		m[j] = tmp
		bt = _eval_map(a, m, wt)
		if bt <= opval + bad*EPS:
			bad = bt
			break
		if bt <= bad * (1 + f/float(n)):
			# print "ACCEPT", bad, bt
			if abs(bt-bad) > 0.1*bad*f/float(n):
				f *= fr
				nobetter = 0
			bad = bt
			nobetter = 0
		else:
			# print "NG", bad, bt
			tmp = m[i]
			m[i] = m[j]
			m[j] = tmp
			nobetter += 1

		if nobetter >= n*n:
			break
	if m[0] > m[-1]:	# Pick one of two equivalent orderings
		m = m[::-1]
	return (m, bad0, bad)



def map_array(a, m):
	return Num.take(Num.take(a, m, axis=0), m, axis=1)


def test():
	a = Num.array([[1, 0, 3, 2], [0, 1, 0, 0], [3, 0, 1, 0], [2, 0, 0, 1]])
	m, b0, b1 = diagonalize(a)
	assert b1<b0
	print m
	print map_array(a, m)


if __name__ == '__main__':
	test()



import pgplot.cpgplot
pl = pgplot.cpgplot


__version__ = "$Revision: 1.1 $"


class plot:
	def __init__(self, dev=None):
		if dev is None:
			dev = 'localhost:0/xw'
		pl.cpgopen(dev)
		self.fresh = 1
		self.ask(0)

	def __del__(self):
		pl.cpgclos()

	def next(self):
		if self.newsize:
			pl.cpgpap(self.width, self.aspect)
			self.newsize = 0
		pl.cpgpage()

	def ask(self, onoff=1):
		pl.cpgask(onoff)

	def size(self, width, aspect):
		if not self.fresh:
			self.width = width
			self.aspect = aspect
			self.newsize = 1
		else:
			pl.cpgpap(width, aspect)
			self.newsize = 0

	def panels(self, w, h):
		pl.cpgsubp(w, h)

	def flush(self):
		pl.cpgupdt()

	def start(self):
		pl.cpgvstd()	# standard viewport



"""This lets you find files with just a path name relative to where
the program's executable code script sits."""

import sys

_scriptname = sys.argv[0]	# Possibly, this is wrong, if we are importing
				# late in execution, after some other module
				# has fiddled with sys.argv.
				# If so, sorry!

import os

__version__ = "$Revision: 1.6 $"


def _searchdir(pathlist, s, access):
	for t in pathlist:
		if t == '':
			t = '.'
		tmp = '%s/%s' % (t, s)
		if os.access(tmp, access) and os.path.isdir(tmp):
			return tmp
	raise RuntimeError, "Can't find %s" % s


def _search(pathlist, s, access):
	for t in pathlist:
		if t == '':
			t = '.'
		tmp = '%s/%s' % (t, s)
		if os.access(tmp, access):
			return tmp
	raise RuntimeError, "Can't find %s" % s


def executable(s, general=True):
	"""Find an executable program.
	It searches first in the directory of the currently executing
	python script.  Optionally, it then looks at PATH.
	If general=False, the function should fail unless
	the executable is in the same directory as the currently
	executing python script."""
	p = [ os.path.dirname( _scriptname ) ]

	if general:
		p.extend( os.environ['PATH'].split(':') )

	return _search(p, s, os.X_OK)



def module(s, general=True):
	"""Find a module. Returns pathname."""
	p = [ os.path.dirname( _scriptname ) ]

	if general:
		p.extend( sys.path )

	return _search(p, "%s.py" % s, os.R_OK)


def os_prgm(nm, general=1):
	"""Find an operating-system dependent executable program."""
	return executable('bin__' + os.uname()[0] + '/' + nm, general)



def data(s):
	"""Find a data file."""
	p = [ '.', os.path.dirname( _scriptname ) ]
	return _search(p, s, os.R_OK)



def directory(s):
	"""Find a directory."""
	p = [ os.path.dirname( _scriptname ) ]
	return _searchdir(p, s, os.X_OK)




if __name__ == '__main__':
	assert data("find_home.py")



"""This module defines closures.
Closures are functions that have been supplied with some arguments,
but not all of them.
"""



class NotYet:
	"""Just a marker, indicating that a certain argument is not yet
	present, and the actual call should be deferred until later.
	"""

	def __repr__(self):
		return "<BLANK>"


class ArgUnspecifiedError(TypeError):
	def __init__(self, s):
		TypeError.__init__(self, s)



class Closure:
	def __init__(self, fcn, *argv, **kwarg):
		"""Create a closure by memorizing a function and its
		arguments.    If all arguments are supplied, you
		can get the result by calling the closure without arguments.
		Missing arguments are replaced by instances of class NotYet;
		such arguments will need to be supplied when the closure is
		called.

		As an additional feature, you can supply a list of users
		of the function's value.    When the value is (eventually)
		produced, the user function(s) will be called with the
		closure's return value.    Users must be functions of one
		argument.
		"""
		self.fcn = fcn
		self.args = argv
		self.argd = kwarg
		self.deferred = []
		self.to_be_filled = []
		for (i, arg) in enumerate(argv):
			if arg is NotYet:
				self.to_be_filled.append(i)

	def __repr__(self):
		return '%s(%s)' % (str(self.fcn),
					','.join([str(q) for q in self.args]
						 + ["%s=%s" % kv for kv in self.argd.items()])
					)

	def __call__(self, *argv, **argd):
		"""This evaluates the closure to either produce a value
		or raise an exception.
		"""
		lstbf = len(self.to_be_filled)
		if len(argv) < lstbf:
			raise ArgUnspecifiedError, len(argv)
		a = list(self.args)
		for (tmparg,j) in zip(argv, self.to_be_filled):
			a[j] = tmparg
		a.extend( argv[lstbf:] )
		adict = self.argd.copy()
		adict.update(argd)
		rv = self.fcn(*a, **adict)
		for user in self.deferred:
			user(rv)
		return rv

	def partial(self, *argv, **argd):
		"""This evaluates the closure to produce another
		(more complete) closure.    Users are not preserved.
		"""
		a = list(self.args)
		for (tmparg,j) in zip(argv, self.to_be_filled):
			a[j] = tmparg
		lstbf = len(self.to_be_filled)
		a.extend( argv[lstbf:] )
		adict = self.argd.copy()
		adict.update(argd)
		rv = Closure(self.fcn, *a, **adict)
		return rv

	def defer(self, user):
		"""This will call the specified function when the closure
		is completed.   It's a bit of a kluge.
		"""
		self.deferred.append( user )




if __name__ == '__main__':
	def foo(a, b):
		return a+b

	x = Closure(foo)
	assert x(1, 3)==4

	x = Closure(foo, 3)
	assert x(4)==7

	try:
		x(1, 1)
	except TypeError:
		pass
	else:
		print "Whoops! Expected TypeError"
	
	try:
		x()
	except TypeError:
		pass
	else:
		print "Whoops! Expected TypeError"

	assert x(b=7)==10

	try:
		x(a=7)
	except TypeError:
		pass
	else:
		print "Whoops! Expected TypeError"
	

	def bar(a, b=8):
		return a*b
	
	y = Closure(bar)
	assert y(a=1)==8
	assert y(a=1,b=2)==2

	y = Closure(bar, 3, 3)
	assert y()==9

	y = Closure(bar, NotYet, 7)

	try:
		y()
	except TypeError:
		pass
	else:
		print "Whoops! Expected TypeError"
	
	assert y(7)==49
	z = y.partial()
	assert z(7)==49
	z = y.partial(8)
	assert z()==56
	def user1(x):
		raise RuntimeError, str(x)
	z.defer(user1)
	try:
		z()
	except RuntimeError, x:
		assert str(x) == "56"
		pass




"""ERB Perceptual frequency scale."""

import math
import Num

__version__ = "$Revision: 1.6 $"

def f_to_erb(f):
	"""Frequency to ERB band number.
	http://ccrma-www.stanford.edu/~jos/bbt/Equivalent_Rectangular_Bandwidth.html
	"""
	return Num.log(0.00437*f+1)*(21.4/math.log(10.0))

def erb_to_f(e):
	"""ERB number -> frequency
	"""
	return (10.0**(e/21.4)-1)/0.00437


def ebw(e):
	"""Critical bandwidth (in Hz) at frequency e (erbs)."""
	return 0.108*erb_to_f(e) + 24.7


if __name__ == "__main__":
	print f_to_erb(51)
	print f_to_erb(149)




"""Does a robust estimate of covariance.
It doesn't converge, unfortunately, for
vectors of dimension higher than 1.
"""

import Num
import math
import sys
import die

def cov_wt(veclist, weights):
	nvec,vecdim = veclist.shape
	o = Num.zeros((vecdim,vecdim), Num.Float)
	for i in range(nvec):
		Num.add(o, Num.outerproduct(veclist[i], veclist[i])*weights[i], o)
	Num.divide(o, Num.sum(weights), o)
	return o


def integrate_guts(xlow, xhigh, fcn, c, n):
	sum = 0.0
	dx = (xhigh-xlow)/float(n)
	for i in range(n):
		x = xlow + (i+0.5)*dx
		sum += fcn(x, c)
	return sum*dx


def integrate(xlow, xhigh, fcn, c=None, nstart=5, tol=1e-4):
	"""A generally useful function, does the integral of
	fcn(x, c) from xlow to xhigh.
	"""
	passes = 0
	while passes < 10:
		i = integrate_guts(xlow, xhigh, fcn, c, nstart)
		tmp = integrate_guts(xlow, xhigh, fcn, c, nstart*3)
		if abs(tmp-i) < tol:
			return tmp + (tmp-i)/9.0
		nstart *= 3
		i = tmp
		passes += 1
	raise RuntimeError, "Integral doesn't converge"


def wtfunc(q, E):
	Qcut = 2.0
	return (Qcut/(Qcut + q))**E
	# return Num.minimum(1.0, q**(-E))


def wtdim(q, Edim):
	E,vecdim,qxtra = Edim
	q2 = q**2
	return math.exp(-q2/2.0)*wtfunc(q2,E) * q**qxtra


def covariance(veclist, emax=1.0):
	"""Veclist[i] is a vector.
	"""
	echg = 0.25
	E = echg * emax
	nvec, vecdim = veclist.shape
	e0 = integrate(0, 3.0+0.5*vecdim, wtdim, c=(0.0,vecdim, 0),
					nstart = vecdim*2)
	e0qq = integrate(0, 3.0+0.5*vecdim, wtdim, c=(0.0,vecdim, 2),
					nstart = vecdim*2)
	# Qtol = SMALL * NDIM * WIDTH_OF_CHISQ(ndim)_DISTRIBUTION
	qtol = 0.001*nvec*math.sqrt(vecdim)
	weights = Num.ones((len(veclist),), Num.Float)
	efac = 1.0
	qlast = None
	ipass = 0
	while ipass < 1000:
		c = cov_wt(veclist, weights) / efac
		assert c.shape == (vecdim, vecdim)
		print 'c.shape=', c.shape
		print 'c=', c
		eval, evec = Num.LA.eigh(c)
		print 'evals=', eval
		if not Num.alltrue(eval >= 0.0):
			weights = Num.sqrt(weights)
			die.warn('Negative eigenvalue: slowing down.')
			continue
		ichalf = evec/Num.sqrt(eval)
		# print 'ERR=', Num.dot(Num.transpose(ichalf), Num.dot(c, ichalf))
		# print 'ichalf=', ichalf
		vecxf2 = Num.dot(veclist, ichalf)**2
		print 'vecxf2=', vecxf2
		assert vecxf2.shape == (nvec, vecdim)
		weighttmp = wtfunc(vecxf2, E)
		print 'weighttmp=', weighttmp
		weights = Num.prod(weighttmp, axis=1)
		assert weights.shape == (nvec,)
		q = Num.sum(vecxf2, axis=1)
		assert q.shape == (nvec,)
		print 'q=', q
		assert Num.alltrue(q >= 0.0)
		print 'weight=', weights
		if qlast is not None:
			qchg = Num.sum(Num.absolute(q-qlast))
			echg = 1.0/math.sqrt(qchg/qtol + 1.0)
			print 'qchg=', qchg, 'echg=', echg, 'E=', E
			if qchg < qtol and abs(E-emax)<0.01:
				return (c, eval, evec, q)
		qlast = q
		e = integrate(0, 3.0+0.5*vecdim, wtdim, c=(E,vecdim,0),
					nstart = vecdim*2)
		eqq = integrate(0, 3.0+0.5*vecdim, wtdim, c=(E,vecdim,2),
					nstart = vecdim*2)
		efac = (eqq/e0qq)/(e/e0)
		print 'efac=', efac
		E = echg*emax + (1.0-echg)*E
		ipass += 1
		sys.stdout.flush()
	raise RuntimeError, "Covariance does not converge."


if __name__ == '__main__':
	Nvec = 2000
	ND = 10
	veclist = Num.RA.normal(0.0, 1.0, (Nvec,ND))
	covariance(veclist)




"""This module is designed to build makefiles."""

import os
import re
import sys
import tempfile
import datetime
import subprocess
from gmisclib import die
from gmisclib import fiatio
from gmisclib import avio

_debug = False
_log = None
makeflags = []
makefile = None
_makefd = None
_makeprog = ['make']
_did_header = False
DB_format = 'fiatio'


def maketemp(dir='/tmp', prefix='maketemp', suffix=''):
	global makefile, _makefd
	handle, makefile = tempfile.mkstemp(dir=dir, prefix=prefix, suffix=suffix)
	_makefd = os.fdopen(handle, 'w')


def _write(*s):
	global _makefd
	for x in s:
		_makefd.writelines([x, '\n'])


def _header():
	global _did_header
	if _did_header:
		return
	_write("# Makefile produced by gmisclib/makemake.py")
	_write(".SUFFIXES:", "")
	_write(".PHONY: all", "")
	_did_header = True


def setlog(f):
	global _log
	_log = open(f, 'a')


def log(*s):
	"""Log some strings, one per line."""
	for x in s:
		_log.writelines([x, '\n'])
	_log.flush()



_qp = re.compile('[^a-zA-Z0-9_.+,/:-]')
def quote(s):
	def replf(x):
		return '\\%s' % x
	return re.sub(_qp, replf, s)


def var(k, v):
	"""Pass a variable to make."""
	_header()
	_write('%s = %s' % (k, v))


def rule(a, *b):
	"""Writes a rule into a makefile.   All lines except the
	first are indented.
	"""
	_write('', a)
	_write( *['\t%s' % t for t in b] )
	_write('')


def blank():
	_header()
	_write("")
	

def set_debug():
	global _debug
	_debug = True


def finish():
	global _makefd, makefile, makeflags
	_makefd.flush()
	os.fsync(_makefd.fileno())
	_makefd.close()
	if _debug:
		tmp = open(makefile, 'r')
		sys.stdout.writelines(tmp.readlines())
	else:
		args = _makeprog + ['-f', makefile] + makeflags
		print '# calling', args
		rv = subprocess.call(args)
		os.remove(makefile)
		if rv != 0:
			die.info("CALL: %s" % (' '.join(args)))
			die.die("Make fails with %d" % rv)

def date():
	return datetime.datetime.now().ctime()


class FileNotFound(Exception):
	def __init__(self, *s):
		Exception.__init__(self, *s)



def path_to(s):
	for d in os.environ['PATH'].split(':'):
		tmp = os.path.join(d, s)
		if os.access(tmp, os.X_OK):
			return tmp
	raise FileNotFound, s


def set_make_prog(*s):
	global _makeprog
	try:
		path_to(s[0])
	except FileNotFound:
		die.die("Specified make program '%s' not on PATH" % s[0])
	_makeprog = s


def ncpu():
	"""
	@rtype: str   !Not an integer!
	@return: a string representation of the integer number of cores that
		the computer has.
	"""
	n = 0
	for x in open('/proc/cpuinfo', 'r'):
		if x.split(':')[0].strip() == 'processor':
			n += 1
	assert n > 0, "Silly!"
	return str(n)


def read(fn):
	if DB_format == 'avio':
		h, d, c = avio.read_hdc(open(fn, 'r'))
	elif DB_format == 'fiatio':
		h, d, c = fiatio.read(open(fn, 'r'))
	else:
		die.die("Unknown metadata format: %s" % DB_format)
	return h, d



#!/usr/bin/env python


import g_pipe
import math


IB = 0.7

def addcurve( xyee, x_range, y_range, bexp, brms, imopfd, xsz, ysz ):
	if len(xyee) == 0:
		return
	xmin, xmax = x_range
	sx = xsz/float(xmax - xmin)
	ymin, ymax = y_range
	sy = ysz/float(ymax - ymin)
	bs2 = 0.0
	for (x, y, ex, ey, wt) in xyee:
		iex = math.hypot(ex*sx, IB)
		iey = math.hypot(ey*sy, IB)
		area = iex*iey
		b = wt * area**(-bexp)
		bs2 += b*b
	if not ( bs2 > 0.0):
		return
	bfac = brms / math.sqrt(bs2/len(xyee))
	for (x, y, ex, ey, wt) in xyee:
		iex = math.hypot(ex*sx, IB)
		iey = math.hypot(ey*sy, IB)
		area = iex*iey
		b = bfac * wt * area**(-bexp)
		xscaled = (x-xmin)*sx
		yscaled = (y-ymin)*sy
		r = math.sqrt(iex * iey)
		ecc = math.sqrt(1 - min(iex, iey)/max(iex, iey))
		pa = 90
		if abs(iey) < abs(iex):
			pa = 0
		imopfd.writelines('%.2f %.2f %.3f %g %g %d addGaussGal\n'
					% (xscaled, yscaled, b, r, ecc, pa))
		# imopfd.flush()




if __name__ == '__main__':
	H = 600
	W = 900
	si, so = g_pipe.popen2("imop", ['imop', '-write16', '-stdin' ])
	x_range = (0, 1)
	y_range = (0, 1)
	xyee = []
	for i in range(10):
		x = 0.1*i
		y = x - 2*x*x*x + 2*x*x*x*x*x
		ex = 0.05 - 0.02*x
		ey = 0.01 + 0.1*y
		xyee.append( (x, y, ex, ey, 1.0) )
	si.writelines('%d %d alloc\n' % (H, W))
	addcurve(xyee, x_range, y_range, 0.5, 200, si, W, H)
	si.writelines('= foo.fits')
	si.close()
	so.close()
